from .files.yota import Yota, Cue, Sample, Mixtape
from .files.bit import Link, Mp3
from .files.lib import Convert
from .files import cnf
#from .files.yota_fuzz import main as yota_fuzz
#from .files.bit_fuzz import main as bit_fuzz
#from .files.ono import check_ono as ono
from .files.onoObject import MediabyteHashObj


omm = Convert.omm
youtube = Convert.search_to_mixtape
search = Convert.search
history = Convert._search_history("")
#regex = Convert.regex_search

input = Convert.omm
version = Convert.version()
readme = Convert.readme()
docs = Convert.docs
summary = cnf.summary
tutorial = Convert.tutorial


import os, subprocess
import glob
from . import cnf

# Platform-independent re-implementation of dl2.sh

def dl_srt(youtube_url):
    """Takes YouTube URL, downloads .srt subtitles to srt folder,
    returns .srt filename."""

    youtube_hash = youtube_url[-11:]

    def vtt_files():
        """Returns .vtt filenames in folder."""
        files = os.listdir()
        vtt_files = []
        for file in files:
            if str(file).endswith('vtt'):
                vtt_files.append(str(file))
        return vtt_files
    
    # build path to srt folder
    
    srt_path = cnf.srt_folder_path + cnf.os_sep + '*.srt'

    filenames = glob.glob(srt_path)

    # check file cache first
    for filename in filenames:
        if youtube_hash in filename:
            return filename

    os.chdir(cnf.srt_folder_path)

    if cnf.platform == 'linux':
        shell_str = 'youtube-dl "' + youtube_url + '" --skip-download --write-auto-sub 1> /dev/null 2> /dev/null'
    else:
        shell_str = 'youtube-dl "' + youtube_url + '" --skip-download --write-auto-sub'
    #print('shell_str:', shell_str)
    subprocess.Popen([shell_str], shell=True).wait()
    
    vtt_files_after = vtt_files()
    
    if len(vtt_files_after) > 1:
        ValueError('More than one .vtt files found after download, please remove any .vtt files in the folder ' + srt_path)
    
    elif len(vtt_files_after) == 1:    
        vtt_filename = vtt_files_after[0]
        filename_wo_end = vtt_filename[:-4]
        srt_filename = filename_wo_end  + '.srt'
        # rename the downloaded .vtt to .srt
        os.rename(vtt_filename, srt_filename)

        return srt_filename






import subprocess, sys, urllib
from . import cnf

def ell(programName, *args):
    
    """Takes program name and argument(s), 
    executes in shell."""

    if cnf.platform == 'linux':
        subprocess.Popen([programName, *args])
    
    elif cnf.platform == 'win32':    
        # escape arguments ampersands for Windows
        new_args = []
        for arg in args:
            new_args.append(arg.replace('&','^&'))
        # special case for Windows: Chrome is invoked with 'start chrome'
        if 'chrome' in programName:    
            subprocess.Popen(['start', 'chrome', *new_args], shell=True)
        # try default (Linux) method and hope for program path existence
        else: 
            subprocess.Popen([programName, *new_args], shell=True)






import re
import copy
import hashlib
from . import bno, yno, ono
from . import bit
from . import search_srt as srt
from . import omm_file_parser, sh
from . import lib
import string
import time
import os
import sh as orig_sh



def iframe(self, width=360, pause=False, title=False, center=True):
    """Returns IFrame format."""

    url = '<center><iframe width="360" height="216" title="YYYYY" src="XXXXX" frameborder=0 allowfullscreen></iframe></center>'
    if width != 360:
        url = url.replace('360', str(width))
        height = int(width * 0.6)
        url = url.replace('216', str(height))
    newUrl = url.replace('XXXXX', self.url)
    if title:
        newUrl = newUrl.replace('YYYYY', title)
    else:
        newUrl = newUrl.replace('title="YYYYY"','')
    if pause:
        newUrl = newUrl.replace('autoplay=1','autoplay=0')
    if title:
        new_title = '<center><p>' + repr(self) + '<p>'
        newUrl = newUrl.replace('<center>',new_title)
    if not center:
        newUrl = newUrl[8:-9]
    
    return(newUrl)

def build_html(self):

    if self.title:
        title = self.title
    else:
        my_type = lib.Convert._determine_type(self.omm)
        title = "My" + my_type

    if self.tags:
        html_str = '<a href="' + str(self.url) + '" title="' + " ".join(self.tags) +  '">' + title + '</a>'
    else:
        html_str = '<a href="' + str(self.url) + '">' + title + '</a>'

    try: 
        for item in self.bits:        
            html_str += '  [' + item.html + ']'
    except:
        pass
    
    if isinstance(self, Drip):
        html_str +=  '  ' + '(a)'

    return html_str    


def build_bits(bits):

    bits_list = []
    if len(bits) > 1:
        for item in bits:
            bit_object = bno.main(item)
            try:
                bit_object.title 
            except ValueError:
                bit_object.title = 'myBit'
                bit_object.update()
            bits_list.append(bit_object)
    elif len(bits) == 1:
        bit_object = bno.main(bits[0])
        bits_list.append(bit_object)
    return bits_list

class Mixtape():
    """Takes Sample object, returns single item Mixtape.
    Also produced from Sample object addition: sample_obj + sample_obj."""

    def __init__(self, *subclips):

        content_list = []
        for subclip in subclips:
            content_list.append(subclip)

        self.content = content_list
        self.find = {sample.first_name : sample for sample in self.content}

    def __matmul__(self, search_term):

        result = []
        for item in self:
            if search_term.replace(" ","_") in item.omm:
                result.append(item)
        if len(result) >= 1:
            result_mix = Mixtape(result[0])
            for item in result[1:]:
                result_mix += item
            return result_mix


    def __len__(self):
        """Show length in seconds."""

        # combined_length = 0
        # for sample in self.content:
        #     combined_length += len(sample)

        combined_length = len(self.content)

        return(combined_length)


    def open_tabs(self):
        """Open all samples in tabs in browser."""
        result = []
        for item in self:
            new_url = item.url[:-1] + '0'
            result.append(new_url)
        
        lib.Convert._browser_open(*result)


    def __getitem__(self, given):
        
        if isinstance(given, slice):
            # do your handling for a slice object:
            cut_mix = copy.deepcopy(self.content[given])
            new_mix = Mixtape(cut_mix[0])
            for item in cut_mix[1:]:
                new_mix += item
            return(new_mix)
        elif isinstance(given, int):
            # Do your handling for a plain index
            return(self.content[given])


    def __repr__(self):

        yotas_string = "mediabyte.Mixtape" + " " * 44 + "o." + self.hash() + "\n"
        yotas_string += "-" * 74 + "\n"
        for i, sample in enumerate(self.content):
            if sample.title:
                title = sample.title
            else:
                type_result = str(type(sample)).split(".")[-1][:-2]
                title = "My" + type_result
            yotas_string += str(i) + ". " + title + " " * (60 - (len(title) + len(str(i))) - 1) + "o." + sample.hash() + "\n"

        return(str(yotas_string))


    def __str__(self):
        
        self_str = "\n".join(self.omm())

        return(self_str)


    def html(self, title_string_input=None):
        """Show HTML format string."""

        html_list = [sample.html for sample in self.content]
        if title_string_input:
            title_string_2 = "<h2>" + title_string_input + "</h2> <br> "
        else:
            title_string_2 = "<h2>" + "mediabyte.Mixtape" + "</h2> <br> "

        html_string = title_string_2

        for i, line in enumerate(html_list):
            html_string += str(i) + ". " + line + " <br>"

        return(html_string)


    def append(self, value):

        self.content.append(value)
        self.find[value.first_name] = value


    def __setitem__(self, key, value):

        self.content.__setitem__(key, value)


    def __delitem__(self, key):

        first_name = self.content[key].first_name
        self.content.__delitem__(key)
        del self.find[first_name]


    def __add__(self, value):

        if isinstance(value, Yota) or isinstance(value, Cue) or isinstance(value, Sample) or isinstance(value, bit.Link) or isinstance(value, bit.Mp3):
            self.content.append(value)

        if isinstance(value, Mixtape):
            for item in value.content:
                self.content.append(item)
                self.find[item.first_name] = item
                

        return(self)


    def time(self):
        """Show time in 1h2m3s format."""

        combined_length = 0
        for sample in self.content:
            try:
                combined_length += len(sample)
            except TypeError:
                print('Error: Only Sample-only Mixtapes support the time method at present...')
                return
        if combined_length > (60 * 60):
            hours = int(combined_length / 60 / 60)
            minutes = int(combined_length / 60 - (hours * 60))
            seconds_left = int(combined_length - ((hours * 60 * 60) + (minutes * 60)))
            time_string = str(hours) + 'h' + str(round(minutes, 0)) + "m" + str(seconds_left) + "s"

            return(time_string)

        if combined_length > 59:
            minutes = int(combined_length / 60)
            seconds_left = int(combined_length - (minutes * 60))
            time_string = str(round(minutes, 0)) + "m" + str(seconds_left) + "s"
            return(time_string)


    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(self):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)


    def show(self, title_str=None):
        """Show HTML output in Jupyter Notebook."""

        from IPython.display import HTML, display
        display(HTML(self.html(title_str)))


    def wall_of_tv(self, width=360, titles=False):
        """Returns Wall-of-TV HTML."""

        iframe_collection_str = ""
        for item in self:
            new_url = item.url[:-1] + "0"
            if item.title:
                title = item.title
            else:
                title = ""
            new_str = '<iframe width="' + str(width) + '" height="' + str(int(width * 0.6)) + '" title="' + title + '" src="' + new_url + '" frameborder=0 allowfullscreen></iframe>'
            if titles:
                new_str = '<h3>' + repr(item) + '</h3> <br>' + new_str
            iframe_collection_str += new_str + " "
    
        return(iframe_collection_str)


    def iframe(self, width=360, titles=False, mediabytehash=True, mixtape_oneliner=False):
        """Returns javascript player HTML."""

        iframe_collection_str = ""
        for item in self:
            new_url = item.url[:-1] + "0" + "&enablejsapi=1"
            if item.title:
                title = item.title
            else:
                title = ""
            new_str = '<iframe width="' + str(width) + '" height="' + str(int(width * 0.6)) + '" title="' + title + '" src="' + new_url + '" frameborder=0 allowfullscreen></iframe>'
            if titles:
                new_str = '<h3>' + repr(item) + '</h3> <br>' + new_str
            iframe_collection_str += new_str + " "
    

        javascript_by_jonas = """<!DOCTYPE html>
                                <html lang="en">
                                <head>
                                <meta charset="utf-8">
                                <meta name="viewport" content="width=device-width, initial-scale=1">
                                <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css">
                                <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.4.0/jquery.min.js"></script>
                                <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js"></script>
                                <script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js"></script>
                                <title> {{ mediabyte.title }}</title>
                                </head>
                                <body>

                                <script src="https://www.youtube.com/iframe_api"></script><script>function onYouTubeIframeAPIReady() { window.yotasamples = {}; let i = 0;  for (let ifrm of document.getElementsByTagName("iframe")) {       ifrm.id = "sample" + i;      ifrm.yotaidx = i;      window.yotasamples[i] = new YT.Player(ifrm.id, {events: {"onStateChange": playerStateChange}});        i++;  }}  function playerStateChange(event) {    console.log(event.data);    switch(event.data) {        case 0:            let myidx = event.target.getIframe().yotaidx;            event.target.getIframe().classList.remove("current");if (window.yotasamples[myidx+1]) { window.yotasamples[myidx+1].getIframe().classList.add("current");window.yotasamples[myidx+1].playVideo();} else{window.yotasamples[0].getIframe().classList.add("current")}                        break    }  }document.querySelector("iframe").classList.add("current");</script><input type=button style="position:fixed;bottom:0;left:0" value="Toggle View" onclick="document.documentElement.classList.toggle(this.dataset.targetclass);" data-targetclass=julekalenderview><style>html:not(.julekalenderview) iframe:not(.current) {display:none} </style>

                                </body>
                                </html>

                                """

        javascript_by_jonas = javascript_by_jonas.replace("{{ mediabyte.title }}", "mediabyte.Mixtape Player")

        iframe_collection_str += javascript_by_jonas

        def one_liner(self):
        
            html_str = '<p>\n' + self.__repr__().split("\n")[0].split(" ")[0] + " "
            html_str += '<strong>' + self.__repr__().split("\n")[0].split(" ")[-1] + '</strong> '


            for item in self:
                 title = item.__repr__().split(" ")[0] 
                 mediabytehash = item.__repr__().split(" ")[-1]
                 temp_html_str = title + " " + "<strong>" + mediabytehash + "</strong> "
                 html_str += temp_html_str
            html_str += "</p>"

            return html_str

        if mediabytehash:
            html_oneliner = one_liner(self)
            if mixtape_oneliner:
                iframe_collection_str = html_oneliner + " \n " + self.omm_oneline() + " <br> " + iframe_collection_str    
            else:
                iframe_collection_str = html_oneliner + "  " + iframe_collection_str


        


        return(iframe_collection_str)

    
    def browser_player(self, width=600):
        """Open Mixtape player in browser"""

        self.write_player_html('mediabyte_temp.htm', width=width)
        sh.ell('xdg-open','mediabyte_temp.htm')
        time.sleep(5)
        os.remove('mediabyte_temp.htm')


    def omm(self):
        """Returns omm format string."""

        omm_list = []
        for item in self:
            omm_list.append(item.omm)

        return(omm_list)


    def omm_oneline(self):
        """Returns omm oneline mixtape format."""

        result = ""
        for item in self:
            result += item.omm + ".."
        
        result = result[:-2]

        return(result)


    def write_player_html(self, filename, width=360):

        html = self.iframe(width=width)
        with open(filename,'w') as f:
            f.write(html)
            f.write("\n\n")
        

    def vlc(self):

        def check_mixtape_for_only_yota(mixtape):

            for item in mixtape:
                if not isinstance(item, Yota):
                    return False
            return True

        # check Mixtape only contains Yota objects
        if check_mixtape_for_only_yota(self):
            url_list = []
            for item in self:
                basic_url = 'https://youtube.com/watch?v=' + item.omm[2:13]
                url_list.append(basic_url)
            sh.ell('vlc', *url_list)
        else:
            raise ValueError('only supported for Yota-only Mixtapes')

        vlc_pid = orig_sh.pidof("vlc").strip()
        if vlc_pid:
            def vlc_quitter():
                """Returns an iterator to quit active VLC processes from newest onward"""
                for item in vlc_pid.split(" "):
                    sh.ell("kill", item)
                    yield (f"killed VLC pid: {item}")
            turn_off_vlc = vlc_quitter()
            return turn_off_vlc


    def vlc_chapter_iter(self):
        def vlc_iter():
            for item in self:
                vlc_quiter = item.vlc()
                yield vlc_quiter
        chapter_iter = vlc_iter()
        return chapter_iter


    def srt_search(self, keyword, min_interval=10, sample_length=7):
        i = 0
        myMix = None
        while not myMix:
            myMix = self[i].srt_search(keyword, min_interval=10, sample_length=sample_length)
            if i + 1 == len(self):
                return myMix
            else:
                i += 1
            
        if self[i:]:
            for item in self[i:]:
                result = item.srt_search(keyword, min_interval=10, sample_length=sample_length)
                if result is not None:
                    for item2 in result:
                        myMix += item2
            return myMix



    def write_omm(self, filename):
        with open(filename, 'w') as f:
            for item in self.omm():
                f.write(item)
                f.write("\n")


    def write_omm_oneline(self, filename=None):
        with open(filename, 'w') as f:
            f.write(self.omm_oneline())
            f.write("\n")


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm_oneline().encode())
        temp_hash = m.hexdigest()

        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]

                return(calculated_hash)



    def add_tags(self, tags=[]):
        for item in self:
            try:
                item.tags += tags
            except:
                item.tags = tags
            item.update()
        #return mixtapeObj



class Sample():
    """Takes YouTube URL, time tuple (start in secs, end in secs) title (optional), tags (optional),
    returns yota Sample object."""

    def __init__(self, url, time_start=None, time_end=None, title=None, tags=[], bits=[]):

            youtube_hash_match = re.search(r'[0-9a-zA-Z_\-]{11}', url)   # extract 11-character youtube hash
            self.youtube_hash = youtube_hash_match.group(0)
            self.parent_url = 'https://www.youtube.com/watch?v=' + url
            self.first_name = self.youtube_hash[:3]
            self.middle_name  =  self.youtube_hash[3:7]
            self.last_name = self.youtube_hash[7:]

            if time_start: self.time_start = time_start
            if time_end: self.time_end = time_end
            if title:
                title2 = title.replace("_"," ")
                self.title = title2
            else:
                self.title = ""
            if tags:
                self.tags = tags
            else:
                self.tags = []

            self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=' + str(self.time_start) + '&end=' + str(self.time_end) + '&rel=0' + '&autoplay=1'


            # omm formatting
            self.omm = 'y.' + self.youtube_hash
            self.omm += '.' + self.youtube_time_format(self.time_start)

            if self.title: self.omm += '.' + self.title.replace(" ", "_")
            if self.tags:
                for tag in self.tags:
                    self.omm += '.' + tag
                if self.omm[-1] == '.':
                    self.omm = self.omm[:-1]
            self.omm += '.' + str(self.youtube_time_format(self.time_end))

            if bits:
                self.bits = build_bits(bits)


                    
            else:
                self.bits = []


            self.html = build_html(self)

            # add bits to Sample.html()
            try: 
                for item in self.bits:
                    self.html += '  [' + item.html + ']'
            except:
                pass


            
            try: 
                self.bits
                #print('self.bits: ', self.bits)
                #print('self.omm:', self.omm)
                for item in bits:
                    self.omm += '.' + item
                    #print('self.omm', self.omm)
            
            except:
                pass

            if self.omm[-1] == '.':
                self.omm = self.omm[:-1]
            
        
            imid = self.format(time_format=True, simple_text=True)
            split_items = imid.split("'")
            new_stri = split_items[0] + "'" + split_items[1]
            if self.title:
                new_stri += '.' + self.title.replace(" ","_")  # NB: Why hasn't this been done yet?
            if self.tags:
                for tag in self.tags:
                    new_stri += '.' + tag
            new_stri += "'" + split_items[2] 
            self.beta_format = new_stri

            self.beta_format_2 = self.format(time_format=False, simple_text=True)

    def __len__ (self):
        """Sample length in seconds."""

        self.length = self.time_end - self.time_start

        return(self.length)


    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(self):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)


    def __repr__(self):
        
        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
        else:
            tag_string = ""
        
        if self.title == "":
            title = "MySample"
        else:
            title = self.title

        result = title + tag_string + '  ' + str(self.time())

        if self.bits:
            result += '  ['
            for item in self.bits:
                result += item.title + ', '
        
            result = result[:-2] + ']'
        
        my_length = 60

        if (my_length - len(title) - len(tag_string) - len(self.time()) + 3) < 0:
            title_length = 60 - len(title) - len(tag_string) - len(self.time()) - 6
            new_title = title[:title_length] + "..."
            result = result.replace(title, new_title)


        result += " " * (60 - len(title) - len(tag_string) - len(self.time()) - 3) + "  o." + self.hash()
        
        return(result)


    def __str__(self):
        
        return(self.omm)

    def play(self):
        """Open sample in tab in browser."""

        lib.Convert._browser_open(self.url)



    def update(self):
        """Updates sample 'url' and 'html' parameters."""
        
        self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=' + str(self.time_start) + '&end=' + str(self.time_end) + '&rel=0' + '&autoplay=1'
        self.html = '<a title="' + " ".join(self.tags) + '" href="' + str(self.url) + '">' + str(self.title) + '</a>'

        # omm formatting
        self.omm = 'y.' + self.youtube_hash
        self.omm += '.' + self.youtube_time_format(self.time_start)
        
        if self.title: self.omm += '.' + self.title.replace(" ", "_")
        if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag
            if self.omm[-1] == '.':
                    self.omm = self.omm[:-1]
        self.omm += '.' + str(self.youtube_time_format(self.time_end))

        if self.bits:
            for item in self.bits:
                self.omm += '.' + item.omm

        if self.omm[-1] == '.':
            self.omm = self.omm[:-1]


    def __add__(self, value):
        new_mixtape_obj = Mixtape(self, value)

        return(new_mixtape_obj)

    def __getitem__(self, value):
        if isinstance(value, int):
            if value >= 0:
                if value < len(self):
                    new_sample = copy.deepcopy(self)
                    new_sample.time_start = new_sample.time_start + value
                    new_sample.update()
                    #newSample = omm(new_sample.omm)
                    newSample = new_sample
                    return(newSample)
                else:
                    raise ValueError('Out of range')


    def __sub__(self, small_sample):
        """split first Sample with second Sample, 
        returns a tuple of two Samples,
        first Sample up to second Sample start and 
        first Sample from second Sample end onwards."""

        # test time range fit
        def test_sample_sub_time_range(full_sample, small_sample):
            if full_sample.youtube_hash != small_sample.youtube_hash:
                raise ValueError('different videos cannot be subtracted')
            else:
                if full_sample.time_start > small_sample.time_start:
                    raise ValueError('second sample time_start exceeds first sample time_start')
                else:
                    if full_sample.time_end < small_sample.time_end:
                        raise ValueError('second sample time_end exceeds first sample time_end')
                    else:
                        return(True)
    
        time_ranges_okay = test_sample_sub_time_range(self, small_sample)
        if time_ranges_okay:
            full_copy_1 = copy.deepcopy(self)
            full_copy_2 = copy.deepcopy(self)
            full_copy_1.time_end = small_sample.time_start
            full_copy_2.time_start = small_sample.time_end
            full_copy_1.update()
            full_copy_2.update()
            return(full_copy_1, full_copy_2)
        else:
            raise ValueError('subtraction not supported for these Samples')


    def time(self):
        """Show sample length in 1h2m3s format."""

        own_length_in_seconds = len(self)

        if own_length_in_seconds > 3599:
            hours = int(own_length_in_seconds / 3600)
            own_length_in_seconds = (own_length_in_seconds - (hours * 3600))
        else:
            hours = 0

        if own_length_in_seconds > 59:
            minutes = int(own_length_in_seconds / 60)
            own_length_in_seconds = (own_length_in_seconds - (minutes * 60))
        else:
            minutes = 0

        seconds = own_length_in_seconds
        time_string = ""

        if hours:
            time_string += str(hours) + "h"
        if minutes:
            time_string += str(minutes) + "m"
        if seconds:
            time_string += str(seconds) + "s"

        return(time_string)


    def youtube_time_format(self, seconds):
        """Takes time in seconds, returns YouTube format time code."""

        hours = int(seconds / 3600)
        seconds = seconds - (hours * 3600)
        minutes = int(seconds / 60)
        rest = seconds - minutes * 60
        final_str = ''
        if hours:
            final_str += str(hours) + 'h'
        if minutes:
            final_str += str(minutes) + 'm'
        if seconds:
            if rest != 0:
                final_str += str(rest) + 's'

        return(final_str)
    

    def iframe(self, width=360, pause=False, title=False, center=True):
        """Returns IFrame format."""

        return(iframe(self, width=width, pause=pause, title=title, center=center))


    def format(self, format='h1', center=True, simple_text=False,bold=False, time_format=True, bold_time = False, dot_time_sep=False):
        """Returns HTML formatted Sample representation."""

        def boldness(input_str):
            """Add <b> tags to string (internal method)."""

            new_str = '<b>' + input_str + '</b>'
            return(new_str)

        yota_obj = self
        translate = yota_obj.youtube_time_format
        if time_format:
            myTuple = yota_obj.youtube_hash, translate(yota_obj.time_start), translate(yota_obj.time_end)
        else:
            myTuple = yota_obj.youtube_hash, str(yota_obj.time_start), str(yota_obj.time_end)
        res = ""
        for item in myTuple:
            res += str(item) + "'"
        res = res[:-1] 
        if bold:
            res_str = '<' + format + '>y.' + myTuple[0] + boldness("'" + myTuple[1] + "'" + myTuple[2]) + '</' + format + '>'
        elif bold_time:
            res_str = '<' + format + '>y.' + myTuple[0] + boldness("'") + '</b>' + myTuple[1] + boldness("'") + '</b>' + myTuple[2] + '</b>' + '</' + format + '>'  
        else:
            res_str = '<' + format + '>y.' + myTuple[0] + "'" + myTuple[1] + "'" + myTuple[2] + '</' + format + '>'
        if center:
            res_str = '<center>' + res_str + '</center>'
        if simple_text:
            res_str = ("y." + res) 
        if dot_time_sep:
            res_str = res_str.replace("'",".")
        return(res_str)      


    def vlc(self, full_screen=False):

        lib.Convert.vlc_open(self, full_screen=full_screen)

        vlc_pid = orig_sh.pidof("vlc").strip()
        if vlc_pid:
            def vlc_quitter():
                """Returns an iterator to quit active VLC processes from newest onward"""
                for item in vlc_pid.split(" "):
                    sh.ell("kill", item)
                    yield (f"killed VLC pid: {item}")
            turn_off_vlc = vlc_quitter()
            return turn_off_vlc


    def pad(self, shift_value):
        """Extends the Sample in time, positive integers extends the front and 
        negative integers extends the end."""

        # pad beginning
        if shift_value >= 0:
            sampleCopy = copy.deepcopy(self)
            sampleCopy.time_start = sampleCopy.time_start - shift_value  # pad with 5
            sampleCopy.update()
        # pad ending
        if shift_value < 0:
            sampleCopy = copy.deepcopy(self)
            sampleCopy.time_end = sampleCopy.time_end - shift_value  # is a negative no.
            sampleCopy.update()
        return(sampleCopy)


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)

        

class Cue():

    def __init__(self, yt_hash, title=None, time_start=0, tags=[], bits=[]):

        self.youtube_hash = yt_hash

        self.first_name = self.youtube_hash[:3]
        self.middle_name  =  self.youtube_hash[3:7]
        self.last_name = self.youtube_hash[7:]

        self.yt_time = time_start
        self.time_start = lib.Convert._time_str(time_start)
        self.tags = tags

        self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=' + str(self.time_start) + '&rel=0' + '&autoplay=1'

        if title:
            title2 = title.replace("_", " ")
            self.title = title2
            if self.tags:
                if self.tags[0] == title:
                    self.tags.pop(0)

        else:
            self.title = None



        # omm formatting
        self.omm = 'y.' + self.youtube_hash
        self.omm += '.' + self.youtube_time_format(self.time_start)

        if self.title: self.omm += '.' + self.title.replace(" ", "_")
        if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag

            if self.omm[-1] == '.':
                self.omm = self.omm[:-1]

        self.html = build_html(self)

        if bits:
            #print('bits:', str(bits))
            self.bits = []
            if len(bits) > 1:
                for item in bits:
                    bit_object = bno.main(item)
                    try:
                        bit_object.title 
                    except ValueError:
                        bit_object.title = 'myBit'
                        bit_object.update()
                    self.bits.append(bit_object)
            elif len(bits) == 1:
                bit_object = bno.main(bits[0])
                self.bits.append(bit_object)

                    
        else:
            self.bits = []


            
        try: 
            self.bits
            #print('self.bits: ', self.bits)
            #print('self.omm:', self.omm)
            for item in bits:
                self.omm += '.' + item
                #print('self.omm', self.omm)
        
        except:
            pass

        if self.omm[-1] == '.':
            self.omm = self.omm[:-1]


                # add bits to Yota.html()
        try: 
            for item in self.bits:
                
                self.html += '  [' + item.html + ']'
        except:
            pass



    def play(self):
        """Open sample in tab in browser."""

        lib.Convert._browser_open(self.url)


    def vlc(self, full_screen=False):

        lib.Convert.vlc_open(self, full_screen=full_screen)

        vlc_pid = orig_sh.pidof("vlc").strip()
        if vlc_pid:
            def vlc_quitter():
                """Returns an iterator to quit active VLC processes from newest onward"""
                for item in vlc_pid.split(" "):
                    sh.ell("kill", item)
                    yield (f"killed VLC pid: {item}")
            turn_off_vlc = vlc_quitter()
            return turn_off_vlc




    def __repr__(self):

        tag_string = "  ("                      # build tag string
        for i, tag in enumerate(self.tags):
            tag_string += str(tag)
            if (i + 1) < len(self.tags):
                tag_string += ', '
        tag_string += ')'

        #result = ""
        if self.title:
            title = self.title
        else:
            title = "MyCue"
            
        result = title

        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
        else:
            tag_string = ""
            
        result += tag_string

        if self.bits:
            result += '  ['
            for item in self.bits:
                result += item.title + ', '

            result = result[:-2] + ']'

        my_length = 60

        if (my_length - len(title) - len(tag_string) + 2) < 0:
            title_length = 60 - len(title) - len(tag_string) - 3
            new_title = title[:title_length] + "..."
            result = result.replace(title, new_title)


        result += " " * (60 - len(title) - len(tag_string)) + " o." + self.hash()
        
        return(result)


    def __str__(self):
        
        return(self.omm)


    def __add__(self, value):

        new_mixtape_obj = Mixtape(self)
        new_mixtape_obj += value

        return(new_mixtape_obj)


    def youtube_time_format(self, seconds):
        """Takes time in seconds, returns YouTube format time code."""

        hours = int(seconds / 3600)
        seconds = seconds - (hours * 3600)
        minutes = int(seconds / 60)
        rest = seconds - minutes * 60
        final_str = ''
        if hours:
            final_str += str(hours) + 'h'
        if minutes:
            final_str += str(minutes) + 'm'
        if seconds:
            if rest != 0:
                final_str += str(rest) + 's'

        return(final_str)

    def iframe(self, width=360, pause=False, title=False, center=True):
        """Returns IFrame format."""

        return(iframe(self, width=width, pause=pause, title=title, center=center))


    def update(self):
        """Updates sample 'url' and 'html' parameters."""

        self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=' + str(self.time_start) + '&rel=0' + '&autoplay=1'

        # omm formatting
        self.omm = 'y.' + self.youtube_hash + '.' + str(self.youtube_time_format(self.time_start))
        
        if self.title: self.omm += '.' + self.title.replace(" ", "_")
        if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag
            
            if self.omm[-1] == '.':
                self.omm = self.omm[:-1]

        if self.bits:
            for item in self.bits:
                self.omm += '.' + item.omm

        if self.omm[-1] == '.':
            self.omm = self.omm[:-1]



    def shift(self, shift_value):
        """Shift the Cue start time, positive integers moves start time forward and 
        negative integers moves it back."""

        # move start time forward
        if shift_value >= 0:
            CueCopy = copy.deepcopy(self)
            CueCopy.time_start = CueCopy.time_start + shift_value
            CueCopy.update()
        # move start time back
        if shift_value <= 0:
            if (shift_value * -1) < self.time_start: # check shift_value not exceeding Sample range
                CueCopy = copy.deepcopy(self)
                CueCopy.time_start = CueCopy.time_start + shift_value  # is a negative no.
                CueCopy.update()
            else:
                raise ValueError('Out of range')
        return(CueCopy)


    def to_sample(self, omm, add=0, time_end_str=None):
    
        cue_copy = copy.deepcopy(self)
        
        if add:
            new_omm = cue_copy.omm + '.' + str(cue_copy.time_start + add) + 's'
            new_sample = omm(new_omm)
            return(new_sample)

        elif time_end_str: 
            new_omm = cue_copy.omm + '.' + time_end_str
            new_sample = omm(new_omm)
            return(new_sample)
        

    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)


    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(Yota):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)



class Yota():

    def __init__(self, yt_hash, tags=[], title=None, bits=[]):

        self.youtube_hash = yt_hash

        self.first_name = self.youtube_hash[:3]
        self.middle_name  =  self.youtube_hash[3:7]
        self.last_name = self.youtube_hash[7:]
        if tags:
            self.tags = tags
        else:
            self.tags = None

        if title:
            title2 = title.replace("_", " ")
            self.title = title2
            if self.tags:
                if self.tags[0] == title:
                    self.tags.pop(0)

        else:
            self.title = None


        self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=0' + '&rel=0' + '&autoplay=1'
  
        if self.tags and self.title:
            self.html = '<a href="' + str(self.url) + '" title="' + " ".join(tags) +  '">' + str(self.title) + '</a>'
  
        elif self.title:
            self.html = '<a href="' + str(self.url) + '">' + str(self.title) + '</a>'

        else:
            self.html = '<a href="' + str(self.url) + '" title="' + str(self.tags) + '"> MyYota</a>'


        if bits:
            self.bits = []
            if len(bits) > 1:
                for item in bits:
                    bit_object = bno.main(item)
                    try:
                        bit_object.title 
                    except ValueError:
                        bit_object.title = 'myBit'
                        bit_object.update()
                    self.bits.append(bit_object)
            elif len(bits) == 1:
                bit_object = bno.main(bits[0])
                self.bits.append(bit_object)
                
        else:
            self.bits = []

        # add bits to Yota.html()
        try: 
            for item in self.bits:
                
                self.html += '  [' + item.html + ']'
        except:
            pass


        # omm formatting
        self.omm = 'y.' + self.youtube_hash

        if self.title: self.omm += '.' + self.title.replace(" ", "_")
        if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag

        try: 
            self.bits
            #print('self.bits: ', self.bits)
            #print('self.omm:', self.omm)
            for item in bits:
                self.omm += '.' + item
                #print('self.omm', self.omm)
            
        except:
            pass

        if self.omm[-1] == '.':
            self.omm = self.omm[:-1]
            #self.update()


    def play(self):
        """Open sample in tab in browser."""

        lib.Convert._browser_open(self.url)


    def vlc(self, full_screen=False):
        """Play Yota in VLC Player."""

        lib.Convert.vlc_open(self, full_screen=full_screen)

        vlc_pid = orig_sh.pidof("vlc").strip()
        if vlc_pid:
            def vlc_quitter():
                """Returns an iterator to quit active VLC processes from newest onward"""
                for item in vlc_pid.split(" "):
                    sh.ell("kill", item)
                    yield (f"killed VLC pid: {item}")
            turn_off_vlc = vlc_quitter()
            return turn_off_vlc


    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(Yota):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)


    def __repr__(self):
        """Defines internal print() format (internal method)."""

        tag_string = "  ("                      # build tag string
        if self.tags:
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '

        tag_string += ')'

        #result = ""
        if self.title:
            title = self.title
        else:
            title = "MyYota"
        
        result = title

        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
        else:
            tag_string = ""
        
        result += tag_string

        if self.bits:
            result += '  ['
            for item in self.bits:
                result += item.title + ', '

            result = result[:-2] + ']'

        my_length = 60

        if (my_length - len(title) - len(tag_string) + 2) < 0:
            title_length = 60 - len(title) - len(tag_string) - 3
            new_title = title[:title_length] + "..."
            result = result.replace(title, new_title)


        result += " " * (my_length - len(title) - len(tag_string)) + " o." + self.hash()
        
        return(result)


    def __str__(self):
        """Returns omm format string (internal method)."""

        return(self.omm)


    def __add__(self, value):

        new_mixtape_obj = Mixtape(self)
        new_mixtape_obj += value

        return(new_mixtape_obj)


    def iframe(self, width=360, pause=False, title=False, center=True):
        """Returns IFrame format."""

        return(iframe(self, width=width, pause=pause, title=title, center=center))


    def update(self):
        """Updates sample 'url' and 'html' parameters."""

        self.url = 'https://www.youtube.com/embed/' + self.youtube_hash + '?start=0' + '&rel=0' + '&autoplay=1'
  
        # omm formatting
        self.omm = 'y.' + self.youtube_hash
          
        if self.title: self.omm += '.' + self.title.replace(" ", "_")
        if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag

        if self.bits:
            for item in self.bits:
                self.omm += '.' + item.omm

        if self.omm[-1] == '.':
            self.omm = self.omm[:-1]


    def srt_search(self, keyword, sample_length=7, min_interval=10, mixtape=True):
        """Takes search term, optional sample length and minimum clip length (for merging nearby results), 
        returns Mixtape of results (optionally list of urls), set sample_length=0 to get Cue results."""


        def cue_str_to_sample(cue_str, sample_length):
            myCue = lib.Convert.omm(cue_str)
            time_in_question = myCue.time_start
            end_time = time_in_question + sample_length
            end_time_yt_format = myCue.youtube_time_format(end_time)
            mySampleStr = myCue.omm + "." + end_time_yt_format
            mySample = lib.Convert.omm(mySampleStr)
            return mySample

        result = srt.main(keyword, self.youtube_hash, min_interval = min_interval)
        if mixtape:
            if result:
                if sample_length == False:
                    myCue = lib.Convert.omm(result[0])
                    ono.add_to_hash_dict(myCue.omm)
                    myMix = Mixtape(myCue)
                else:
                    mySample = cue_str_to_sample(result[0], sample_length)
                    myMix = Mixtape(mySample)

                for item in result[1:]:
                    if sample_length == False:
                        myCue = lib.Convert.omm(item)
                        # add to hash_dict
                        ono.add_to_hash_dict(myCue.omm)
                        myMix += myCue
                    else:
                        mySample = cue_str_to_sample(item, sample_length)
                        myMix += mySample
                # add Mixtape to hash_dict
                lib.Convert.omm(myMix.omm_oneline())
                return myMix

        return result


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)


class Drip():
    
    def __init__(self, amazon_hash, title=None, tags=[]):
        
        self.hash = amazon_hash
        self.url = 'https://www.amazon.com/dp/' + amazon_hash + '/'

        self.first_name = amazon_hash
        self.tags = tags

        self.omm = 'a.' + amazon_hash
        if title:
            self.omm += '.' + title.replace(' ','_')
            self.title = title
        else:
            self.title = ""
        if tags:
            for item in tags:
                self.omm += '.' + item
                
        self.html = build_html(self)
    
    def __repr__(self):
        return self.omm
    



import re, json, requests
from more_itertools import unique_everseen
from . import yn, sh, yno, bno
from . import yota
from . import omm_file_parser
from . import online_omm_parser
from . import cnf
from . import ono
from . import onoObject


class Convert():


    def tutorial(dark=True):
        """Open tutorial Mixtape in browser"""

        if dark == True:
            sh.ell("xdg-open","http://v1d.dk/omm/mixtape_tutorial.png")
        else:
            sh.ell("xdg-open","http://v1d.dk/omm/mixtape_tutorial_light.png")
            

    def readme():
        """Returns README.md Markdown file content."""

        file_path = cnf.package_path + cnf.os_sep + 'README.md'
        with open(file_path, 'r') as f:
            file_content = f.read()
            
            return file_content


    def _search_history(search_term, exclusive_terms=[], mixtape=False):
        """Search omm() parsing history, exclusive_terms takes 
        a list of regex strings to exclude, returns list of result strings, 
        optionally Mixtape."""

        # create a MediabyteHashObj (with current parsing history)
        o = onoObject.MediabyteHashObj()
        result = []
        for item in o: 
            if search_term.lower() in o[item].lower() or search_term.lower() in item.lower():
                exclusive_match = False
                if exclusive_terms:

                    for exclusive_term in exclusive_terms:
                        #print(f'o[item]: {o[item]}, ex_term: {exclusive_term}')
                        m = re.search(exclusive_term, o[item])
                        if m:
                            exclusive_match = True
                # break loop cycle if exclusive match
                if exclusive_match == True:
                    #print(f'dropping {o[item]}')
                    continue

                result.append(o[item])
                if mixtape:
                    # append to Mixtape
                    try:
                        myMix += Convert.omm(o[item])
                    # create new Mixtape
                    except:
                        myMix = yota.Mixtape(Convert.omm(o[item]))

        if mixtape:
            try:
                return myMix
            except:
                pass
        else:
            return result


    def sync():
        """Upload local hash_dict to server."""
        filename = cnf.hash_dict_path
        with open(filename, 'r') as f:
            hash_dict = json.load(f)

        resp = requests.post('http://taext.pythonanywhere.com/post', json=hash_dict)
        if resp:
            print(resp.content.decode())
        else:
            print(resp)


    def filter(omm_str, contains=[], does_not_contain=[]):
        """Takes omm string, contains and does_not_contain lists, returns Bool"""
        
        def test_for_tag(tag, omm_str):
            """Test for tag in omm string"""
            m = re.search('[\.^]' + tag + '(\.|$)', omm_str)
            if m: return(True)
            else: return(False)
        
        # OR filtering       (using contains list)
        for item in contains:
            contains_tag_check = test_for_tag(item, omm_str)            
            if contains_tag_check:
                # NOT filtering      (using does_not_contain list)
                for item in does_not_contain:
                    tag_exclusion_check = test_for_tag(item, omm_str)
                    if tag_exclusion_check:
                        return False
                # return OR filtering result
                return contains_tag_check                


    def youtube_takeout_to_omm(youtube_takeout_html_filename, omm_filename):
        """Takes local YouTube watch history HTML filename, writes .omm Mixtape file."""
        
        with open(youtube_takeout_html_filename,'r') as f:
            file_content = f.read()
        m = re.findall('videoId\":"(\S{11})\"', file_content)
        non_results = list(unique_everseen(m))
        m2 = re.findall('v=([a-zA-Z0-9\_\-]{11})', file_content)
        results2 = list(unique_everseen(m2))
        
        with open(omm_filename,'w') as f:
            for item in results2:

                f.write('y.' + item)
                f.write('\n')


    def youtube_html_to_omm(youtube_html_filename, omm_filename):
        """Takes saved YouTube playlist HTML filename, writes .omm mixtape file."""
        
        with open(youtube_html_filename,'r') as f:
            file_content = f.read()
        m = re.findall('videoId\":"(\S{11})\"', file_content)
        non_results = list(unique_everseen(m))
        m2 = re.findall('v=([a-zA-Z0-9\_\-]{11})', file_content)
        results2 = list(unique_everseen(m2))
        
        with open(omm_filename,'w') as f:
            for item in results2[21:]:
                # # NB: Hacky, filtering y.y results (because of faulty .y. mixtape splitting)
                # m = re.search('^y', item)
                # if not m:
                f.write('y.' + item)
                f.write('\n')


    def docs():
        """Open online documentation in browser."""
        sh.ell('google-chrome','https://github.com/taext/mediabyte/blob/master/user_guide/README.md')


    def _time_str(time_str):
        """Takes '2h5m3s' format time string, returns value in seconds (internal method)."""

        m = re.search('([0-9]+)s', time_str)   # match seconds
        m2 = re.search('([0-9]+)m', time_str)  # match minutes
        m3 = re.search('([0-9]+)h', time_str)  # match hours

        if m:
            seconds = m.group(1)
        else: seconds = 0

        if m2:
            minutes = m2.group(1)
        else: minutes = 0

        if m3:
            hours = m3.group(1)
        else: hours = 0

        seconds_total = int(seconds) + (int(minutes) * 60) + (int(hours) * 3600)

        return(seconds_total)


    # def code_length():
    #     """Show total code line count."""

    #     f = open('mediabyte.py','r')
    #     lines = f.readlines()

    #     lines_of_code = []
    #     for line in lines:

    #         if len(line) > 5:
    #             m = re.search('^\s*#', line)    # check for comment line to be excluded
    #             m2 = re.search('\"\"\"', line)  # check for docstrings to be excluded
    #             if m or m2:
    #                 pass
    #             else:
    #                 lines_of_code.append(line)

    #     return(len(lines_of_code))


    def version():
        """Show version number."""
        # get version from cnf.py
        version_string = cnf.version_number

        return(version_string)


    def methods():
        """Show methods and parameters."""

        result = []
        for item in dir(Convert):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)


    def _browser_open(*input_url):
        """Open URL in Chrome tab (internal method)."""

        sh.ell('google-chrome', *input_url)


    # def whats_new():
    #     """Show What's New comment"""

    #     resource_package = 'mediabyte'  # Could be any module/package name
    #     resource_path = '/'.join(('files', 'parse.py'))  # Do not use os.path.join()
    #     file_lines = str(pkg_resources.resource_string(resource_package, resource_path)).split("\n")
    
    #     for item in file_lines[:100]:
    #         m = re.search('Whats New: (.*?)\\n', item)
    #         if m:
    #             return m.group(1)


    def _determine_type(item):
        """Takes yno string, returns yota object type."""

        res = yno.main(item)
        time_code_list = res[0] # get time code occurrences
        m = re.findall('\.\.', item) # test for Mixtape
        if m: 
            if len(m) >= 1:  # '..' match means Mixtape
                answer = 'Mixtape'
        # if no Mixtape match
        elif len(time_code_list) == 1: # 1 time code means Cue
            answer = 'Cue'
        elif len(time_code_list) == 2: # 2 time cods means Sample
            answer = 'Sample'
        elif len(time_code_list) == 0: # 0 time code means Yota
            answer = 'Yota'
        else:
            raise ValueError('unknown format')
            answer = None
        return(answer)


    def vlc_open(self, full_screen=False):
            """Play Sample in VLC player (internal method)."""
            
            obj_type = Convert._determine_type(self.omm)

            url = self.url
            if obj_type == 'Sample':
                start_time = '--start-time=' + str(self.time_start)
                stop_time = '--stop-time=' + str(self.time_end)
                if full_screen:
                    sh.ell('vlc', url, start_time, stop_time, '--fullscreen')
                else:                    
                    sh.ell('vlc',url, start_time, stop_time)
            
            if obj_type == 'Cue':
                start_time = '--start-time=' + str(self.time_start)
                sh.ell('vlc',url, start_time)

            if obj_type == 'Yota':
                sh.ell('vlc',url)


    # def url_file_to_mixtape(filename):
    
    #     with open(filename) as f:
    #         urls = f.readlines()

    #     # remove newlines
    #     urls2 = []
    #     for item in urls:
    #         urls2.append(item.rstrip())
    #     # start mixtape with first url    
    #     if len(urls2) > 0:
    #         mixtape = Convert.omm(urls2[0])
    #     # build mixtape with remaining urls    
    #     if len(urls2) > 1:
    #         for item in urls2[1:]:
    #             mixtape += Convert.omm(item)
                
    #     return mixtape


    def url_list_to_mixtape(url_list):
    
        urls = url_list
        # remove newlines
        urls2 = []
        for item in urls:
            urls2.append(item.rstrip())
        # start mixtape with first url    
        if len(urls2) > 0:
            mixtape = Convert.omm(urls2[0])
        # build mixtape with remaining urls    
        if len(urls2) > 1:
            for item in urls2[1:]:
                mixtape += Convert.omm(item)
                
        return mixtape


    def search_to_mixtape(search_string, clip_length=0, time_end_str="", time_start_str=""):
        """Takes yn format YouTube string, returns Mixtape of Yotas, 
        optional clip_length argument returns Mixtape of Samples."""
    
        urls = yn.return_results(search_string)
        myMix = Convert.url_list_to_mixtape(urls)

        # for i, item in enumerate(myMix):
        #     item.title = search_string.capitalize() + ' - ' + str(i+1)
        #     item.update()

        if clip_length:
            first_sample = myMix[0].to_sample(add=clip_length)
            new_mix = yota.Mixtape(first_sample)
            for item in myMix[1:]:
                new_sample = item.to_sample(add=clip_length)
                new_mix += new_sample
            myMix = new_mix

        elif time_end_str:

            if not time_start_str:
                time_start_str = '1s'

            first_sample = myMix[0].to_sample(time_end_str=time_end_str, time_start_str=time_start_str)
            new_mix = yota.Mixtape(first_sample)
            for item in myMix[1:]:
                new_sample = item.to_sample(time_end_str=time_end_str, time_start_str=time_start_str)
                new_mix += new_sample
            myMix = new_mix

        if isinstance(myMix, yota.Yota) or isinstance(myMix, yota.Cue) or isinstance(myMix, yota.Sample):
            ono.add_to_hash_dict(myMix.omm)
        if isinstance(myMix, yota.Mixtape):
            if len(myMix) > 1:
                # add the Mixtape itself to hash_dict
                ono.add_to_hash_dict(myMix.omm_oneline())
            # add first Mixtape item to hash_dict
            ono.add_to_hash_dict(myMix[0].omm)
            for item in myMix[1:]:
                # add the following items to hash_dict
                ono.add_to_hash_dict(item.omm)


        return myMix


    def search_to_mixtape_player(search_string, filename, clip_length=False, time_end_str=False):
        """Takes yn format YouTube string and filename,
        writes Mixtape player HTML file."""

        mix_obj = Convert._search_history_to_mixtape(search_string, clip_length, time_end_str=time_end_str)

        mix_obj.write_player_html(filename)


    def omm(omm_str, remember=True):
        """Takes OMM format string, returns MediaByte object."""
        
        def parse_omm_mixtape(inp):
            """Split mixtape string into list (split on .y)."""

            newInp = inp.split('.y.')
            omm_lines = [newInp[0]]
            for item in newInp[1:]:
                omm_lines.append("y." + item)
            return(omm_lines)


        def build_omm_mixtape(mix_list):
            """Takes yno string list from parse_omm_mixtape(), returns Mixtape object."""

            myMixtape = yota.Mixtape(parse_object(mix_list[0]))
            for item in mix_list[1:]:
                yotaObject = parse_object(item)
                myMixtape += yotaObject
            return(myMixtape)


        def parse_object(in_obj):
            """Takes yno string, returns yota object: Sample, Cue, Yota or Mixtape."""
            
            # test for bit
            m = re.search('^b\.', in_obj)
            if m:
                myBit = bno.main(in_obj)
                return(myBit)

            type_result = determine_type(in_obj)
            parsing_res = yno.main(in_obj)

            if len(parsing_res[1]) > 0:
                title = parsing_res[1][0]
            else:
                title = ""

            


            if type_result == 'Sample':
                # Sample hash bit object(s)
                if len(parsing_res[4]) > 0:
                    mySample = yota.Sample(url=parsing_res[2], time_start=Convert._time_str(parsing_res[0][0]), time_end=Convert._time_str(parsing_res[0][1]), title=title, tags=parsing_res[3], bits=parsing_res[4])
                else:
                    mySample = yota.Sample(url=parsing_res[2], time_start=Convert._time_str(parsing_res[0][0]), time_end=Convert._time_str(parsing_res[0][1]), title=title, tags=parsing_res[3])
                return(mySample)
            if type_result == 'Cue':
                # Cue has bit object(s)
                if len(parsing_res[4]) > 0:
                    myCue = yota.Cue(parsing_res[2], time_start=parsing_res[0][0], title=title, tags=parsing_res[3], bits=parsing_res[4])
                else:
                    myCue = yota.Cue(parsing_res[2], time_start=parsing_res[0][0], title=title, tags=parsing_res[3])
                return(myCue)
            if type_result == 'Yota':
                # Yota has bit object(s)
                if len(parsing_res) == 5:
                    myYota = yota.Yota(parsing_res[2], title=title, tags=parsing_res[3], bits=parsing_res[4])
                else:
                    myYota = yota.Yota(parsing_res[2], title=title, tags=parsing_res[3])
                return(myYota)
            if type_result == 'Mixtape':
                mixtape_lines = parse_omm_mixtape(in_obj)
                myMixtape = build_omm_mixtape(mixtape_lines)
                return(myMixtape)


        def determine_type(item):
            """Takes yno string, returns yota object type."""

            res = yno.main(item)
            time_code_list = res[0] # get time code occurrences
            m = re.findall('\.\.', item) # test for Mixtape
            m2 = re.search('^b\.', item)  # test for Bit.Link
            m3 = re.search('\.mp3\W', item) # test for Bit.Mp3
            # Bit matched
            if m2 and not m3:
                answer = 'bit.Link'
            elif m2:
                answer = 'bit.Mp3'
            elif m: 
                if len(m) >= 1:  # '..' match means Mixtape
                    answer = 'Mixtape'
            # if no Mixtape match
            elif len(time_code_list) == 1: # 1 time code means Cue
                answer = 'Cue'
            elif len(time_code_list) == 2: # 2 time cods means Sample
                answer = 'Sample'
            elif len(time_code_list) == 0: # 0 time code means Yota
                answer = 'Yota'
            else:
                raise ValueError('unknown format')
                answer = None
            return(answer)


        def check_for_full_youtube_url(url):

            """ parse YouTube URL and return Yota object string y.youtubehash."""
            
            # NB: Hacky, correct for updated share link format
            # https://youtu.be/UW_oi30h4vU?t=480 (no hms)
            m = re.search('youtu.be', url)
            m2 = re.search('t=\d+$', url)
            if m and m2:
                url += 's'

            m3 = re.search('time_continue=(\d+)', url)
            if m3:
                m4 = re.search('v=([a-zA-Z0-9\_\-]{11})', url)
                new_str = 'y.' + str(m4.group(1)) + '.' + m3.group(1) + 's'
                return(new_str)
            
            
            if url[-13:-11] == 'v=':
                new_str = 'y.' + url[-11:]
                return(new_str)
            # example: https://www.youtube.com/watch?v=eBVGYdHNUW4
            
            
            m = re.search('t=([\dsmh]+)$', url)
            m2 = re.search('([a-zA-Z0-9\_\-]{11})', url)
            if m:
                new_str = 'y.' + str(m2.group(1)) + '.' + str(m.group(1))
                return(new_str)
            
            

        def check_for_online_omm(input_str):
            # check for .omm input_str ending
            omm_check = re.search('\.omm$', input_str)
            # check for http input_str start
            http_check = re.search('^http', input_str)
            if omm_check and http_check:
                omm_object = online_omm_parser.main(input_str, Convert.omm)
                return(omm_object)


            
        
        def check_for_yota_file(input_str):
            """Takes omm string, checks for .omm ending"""

            m = re.search('\.omm$', input_str)
            if m:
                omm_object = omm_file_parser.main(input_str, Convert.omm)
                return(omm_object)


        def split_yotas(yotas_str):

            m = re.search('\.\.', yotas_str)
            if m: # check not matching y.youtubehash (hash starting with y)
                my_list = yotas_str.split('..')
                new_list = [my_list[0]]
                # for item in my_list[1:]:
                #     new_str = 'y.' + item
                #     new_list.append(new_str)
                return(my_list)
            else:
                return(yotas_str)

        def check_for_ono(input_str):
            # check for o.mediabytehash
            m = re.search('^o\.', input_str)
            if m:
                myObject = ono.check_ono(input_str)
                return(myObject)

        def check_for_bit_mixtape(input_str):
            starts_with_bit_check = re.search('^b\.', input_str)
            bit_mixtape_check = re.search('\.b\.', input_str)
            if starts_with_bit_check and bit_mixtape_check:
                return(True)
        
        def parse_bit_mixtape(input_str):
            my_list = input_str.split('.b.')
            new_list = [my_list[0]]
            for item in my_list[1:]:
                new_str = 'b.' + item
                new_list.append(new_str)
            myMix = bno.main(new_list[0]) + bno.main(new_list[1])
            for item in new_list[2:]:
                myMix += bno.main(item)
            return(myMix)


        # check for online .omm
        result = check_for_online_omm(omm_str)
        if result:
            return(result)


        # check for bit Mixtape
        result = check_for_bit_mixtape(omm_str)
        if result:
            bitMix = parse_bit_mixtape(omm_str)
            return(bitMix)

        # check for ono string (in hash_dict.json)
        result = check_for_ono(omm_str)
        if result is not None:
            return(result)
        # add to hash_dict if not recognized
        else:
            if remember == True:
                # check for b. or y. (to avoid non-mediabyte entries)
                m = re.search('^[by]\.', omm_str)
                if m:
                    ono.add_to_hash_dict(omm_str)


        # check for full YouTube URL, parse to Yota string 
        omm_str = omm_str.strip()
        result = check_for_full_youtube_url(omm_str)
        if result:
            omm_str = result
        # check for .yota file
        file_check = check_for_yota_file(omm_str)
        if file_check:
            omm_str = file_check

        # mixtape handling
        omm_lines = split_yotas(omm_str)
        if isinstance(omm_lines, list):
            myMix = parse_object(omm_lines[0])
            for line in omm_lines[1:]:
                myMix += parse_object(line)
            # add to hash_dict.json
            ono.add_to_hash_dict(myMix.omm_oneline())
            return(myMix)

        m = re.search('^[by]\.', omm_str)
        if m:
            return(parse_object(omm_str))
        else:
            raise ValueError("input doesn't compute")



    def search(search_term, save_filter_as=""):
        """Search history, regex enabled, save filter with save_filter_as"""

        def check_filters_dict(filter_name):
            """Check for saved filter"""

            with open(cnf.filters_path, 'r') as f:
                filters = json.load(f)
            if search_term in filters:
                return filters[search_term]

        history = Convert._search_history("")
        result = []

        filter_check = check_filters_dict(search_term)
        if filter_check:
            search_term = filter_check


        for item in list(set(history)):
            # ignore out old-style Mixtapes
            if '.y.' not in item:
                # ignore y.youtubehash only items
                if not (len(item.split(".")) < 3 and item[:2] == "y."):  # NB: Hacky, needs to be thought through
                    m = re.search(search_term, item)
                    if m:
                        result.append(item)
        if len(result) > 0:
            myMix = yota.Mixtape(Convert.omm(result[0]))
        for item in result[1:]:
            myMix += Convert.omm(item)

        if save_filter_as:
            with open(cnf.filters_path, 'r') as f:
                filters = json.load(f)
            if save_filter_as not in filters:
                filters[save_filter_as] = search_term
                with open(foldername, 'w') as f:
                    json.dump(filters, f)
            

        if len(result) > 0:
            return myMix


from . import bit
import re


"""
                    yNo Bit String Parser
                       March 10th 2019
                        by d@v1d.dk

What's New: porting from yno.py

"""


def get_time_codes(tag_str):

    """Extract YouTube time codes from CLI string."""

    mylist = tag_str.split(".")
    mytimes = []
    newtimes = []
    for item in mylist:

        yt_time_match = re.search(r'^(\d{1,2}[hms]){1,3}$', item)
        if yt_time_match:
            #print('time code match:', yt_time_match.group(0))
            mytimes.append(yt_time_match.group(0))
        else:
            newtimes.append(item)
#    print('time:', mytimes, ".".join(newtimes))
    return(mytimes, ".".join(newtimes))


def get_titles(tag_str):

    """Extract titles from CLI string."""

    mylist = tag_str.split(".")
    mytitles = []
    newtitles = []
    for i, item in enumerate(mylist):
        title_match = re.search('^[A-Z]', item)  # match on uppercase first character
        if title_match:
            # check not matching bitly hash
            if i != 1:
                mytitles.append(item)
            else:
                newtitles.append(item)
        else:
            newtitles.append(item)
    #if len(mytitles) == 0:
    #    mytitles = ['MyBit']
    return(mytitles, ".".join(newtitles))


def get_bitly_hash(tag_str):

    """Extract Bitly URL or hash from string."""

    mylist = tag_str.split(".")

    newtimes = []
    result = ""

    if mylist[0] == 'b':
        #print(tag_str)
        #print('get_bitly_hash_result:', mylist[1], 'rest:', ".".join(mylist[2:]))
        return(mylist[1], ".".join(mylist[2:]))
        #m = re.search('b\.([^\.]+)\.?(.+$)', tag_str)
        #print('Bit object matched...')
        #print(tag_str[2:9])
        #print(tag_str[10:])
        #return(tag_str[2:9], tag_str[10:])
        print(m.group(1), m.group(2))
        return(m.group(1), m.group(2))


def parse_element_types(str_input):
    """Parse string for time codes, titles and URL."""
    bitly_hash, rest = get_bitly_hash(str_input)
    time_codes, rest2 = get_time_codes(rest)
    titles, rest3 = get_titles(rest2)
    
    raw_tags = rest3.split(".")
    tags = []
    for item in raw_tags:
        if len(item) > 0:
            tags.append(item)

    #print('tags:',tags)

    return(time_codes, titles, bitly_hash, tags)


def print_report(result_in):
    """Print time code, title, url and tags."""
    if result_in[0]:
        print("Time Codes: " + " ".join(result_in[0][:]))
    if result_in[1]:
        print("Titles: " + " ".join(result_in[1]))
    if result_in[2]:
        print("YouTube URL: " + result_in[2])
    if result_in[3]:
        print("Tags: " + " ".join(result_in[3]))



def main(yno_string):
    """Main function, takes a yNo format string, returns string result."""
    result = parse_element_types(yno_string)
    resObj = build_bit_object(result)
    return resObj


def build_bit_object(result_list):
    """Builds and returns Bit object."""
    times_list = result_list[0]
    title_list = result_list[1]
    bitly_hash = result_list[2]
    tags_list = result_list[3]

    if len(title_list) == 0:
        #title = 'MyBit'
        title = None
    else:
        title = result_list[1][0]

    # Stream object detected
    if 'm3u8' in tags_list:
        myStream = bit.Stream(bitly_hash=bitly_hash, title=title, tags=tags_list)
        return myStream

    # Sample detected (two time codes)
    if len(times_list) == 2 and 'mp3' in tags_list:

        timesinsec_list = []
        for item in times_list:
            timesinsec_list.append(time_str(item))

        try:
            newBitMp3 = bit.Mp3(bitly_hash, title=title, time_start=timesinsec_list[0], time_end=timesinsec_list[1], tags=tags_list)
            return newBitMp3
        except:
            raise ValueError("Building BitMp3 object failed (exactly two time codes needed")

    elif len(times_list) == 1 and 'mp3' in tags_list:

        timesinsec_list = []
        #for item in times_list[]:
        timesinsec_list.append(time_str(times_list[0]))

        try:
            newBitMp3 = bit.Mp3(bitly_hash, title=title, time_start=timesinsec_list[0], tags=tags_list)
            return newBitMp3
        except:
            raise ValueError("Building BitMp3 object failed (exactly two time codes needed")

    
    elif times_list and not 'mp3' in tags_list:
        raise ValueError('Time code matched but content type not indicated (presently mp3 only)')


    elif 'mp3' in tags_list:
        try:
            newBitMp3 = bit.Mp3(bitly_hash, title=title, tags=tags_list)
            return newBitMp3
        except:
            raise ValueError("Building BitMp3 object failed (no time codes)")


    elif len(times_list) == 0:

        try:
            #print('result_list[1][0]:', result_list[1][0])
            #print('bitly_hash:', bitly_hash, 'title:', title, 'tags:', tags_list)
            newBitMp3 = bit.Link(bitly_hash, title=title, tags=tags_list)
            return newBitMp3
        except:
            raise ValueError("Building Bit object failed")

        

def mp3_or_not(str_in_question):
    """Test if first tag is 'mp3'."""
    #tags = parse_element_types(str_in_question)[3]
    m = re.search('\.mp3', str_in_question)
    if m:
    #if tags[0] == 'mp3':
        return(True)


# def main(bno_str):

#     # MP3 tag found
#     if mp3_or_not(bno_str):
#         #print('.mp3 matched')
#         BitMp3 = go(bno_str)
#         #BitMp3 = build_bit_object(result)

#     else:
#         Bit = go(bno_str)
#         BitMp3 = Bit
#         #print(result)
#         #BitMp3 = Bit.Bit(result)

#     return(BitMp3)


# def youtube_time_format(seconds):
#     """Takes time in seconds, returns YouTube format time code."""

#     hours = int(seconds / 3600)
#     seconds = seconds - (hours * 3600)
#     minutes = int(seconds / 60)
#     rest = seconds - minutes * 60
#     final_str = ''
#     if hours:
#         final_str += str(hours) + 'h'
#     if minutes:
#         final_str += str(minutes) + 'm'
#     if seconds:
#         final_str += str(rest) + 's'

#     return(final_str)


def time_str(time_str):
        """Takes '2h5m3s' format time string, returns value in seconds (internal method)."""

        m = re.search('([0-9]+)s', time_str)   # match seconds
        m2 = re.search('([0-9]+)m', time_str)  # match minutes
        m3 = re.search('([0-9]+)h', time_str)  # match hours

        if m:
            seconds = m.group(1)
        else: seconds = 0

        if m2:
            minutes = m2.group(1)
        else: minutes = 0

        if m3:
            hours = m3.group(1)
        else: hours = 0

        seconds_total = int(seconds) + (int(minutes) * 60) + (int(hours) * 3600)

        return(seconds_total)



#!/home/dd/anaconda3/bin/python
# yn - YouTube first result new (arbitrary no. results & piping) 
# v0.53 (March 8th 2019)

# Searches youtube.com and returns search result URLs.

# What's New: remove commented-out code

import sys, requests, re
from . import yt_stepper
from . import cnf



def return_results(args, count=1):
    
    """Takes args string, returns YouTube search result(s),
    1-2 digit integer values in args interpreted as count, 
    default count 1."""

    # NB: CHECKS FOR OVERRIDDING INTEGER VALUE count
    # IN SEARCH TERMS (aggresively, any integer 
    # surrounded by whitespace)

    m = re.search(' (\d{1,2})$', args)
    if m:
    
        count = int(m.group(1))
        args = args[:-(len(m.group(1))+1)]  # remove result count arg

    m2 = re.search(', (today|week|month|year)', args)
    if m2:
        today = True
        args = args.replace(m2.group(0),'')
    else:
        today = False

    search_string = args.replace(" ","+")
    youtube_string = 'https://www.youtube.com/results?search_query=' + search_string
    if today:
        youtube_string += '%2C+' + m2.group(1)
        
    urls = []
    while count > 0:
        # build 'Yota v8.2.4' style user-agent
        header_str = 'Yota ' + cnf.version_number
        headers = {'User-Agent': header_str}
        r = requests.get(youtube_string, headers=headers)
        m = re.findall('href=\"\/watch\?v=([a-zA-Z_\-0-9]{11})\"', r.text)
        uniqued = []
        for item in m:
            if item not in uniqued:
                uniqued.append(item)
        m_unique = uniqued

        [urls.append('https://www.youtube.com/watch?v=' + url) for url in (m_unique)[:count]]

        count = count - len(m_unique)
        youtube_string = yt_stepper.next_page(youtube_string)

    return(urls)


# if __name__ == '__main__':

#     if len(sys.argv) == 1:       # no arguments, piping mode
#         args = sys.stdin.read()  # if no piped input,
#         result = return_results(args)  # reads from keyboard
    
#     else:
#         args = ""
#         for item in sys.argv[1:]:
#             args = args + item
#             args += " "
#         args = args.rstrip()
#         if args:
#             result = return_results(args)

#     for url in result:
#         print(url)




#!/home/dd/anaconda3/bin/python
import os, re, json, hashlib, string
from . import lib
from . import cnf


hash_dict_filename = cnf.package_path + cnf.os_sep + 'files' + cnf.os_sep + 'hash_dict.json'

if not os.path.isfile(hash_dict_filename):
    tempDict = {}
    with open(hash_dict_filename, 'w') as f:
        json.dump(tempDict, f)


# def build_hash_dict():
    
#     hash_dict = {}

#     for filename in omm_filenames:
#         tempMix = lib.Convert.omm(filename)
#         tempHash = tempMix.hash()
#         hash_dict[tempHash] = tempMix.omm_oneline()
#         for yotaObject in tempMix:
#             tempHash = yotaObject.hash()
#             hash_dict[tempHash] = yotaObject.omm

#     return hash_dict


def write_hash_dict(hash_dict):

    with open('hash_dict.json','w') as f:
        json.dump(hash_dict, f)


def load_hash_dict():
    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)
    return(hash_dict)


def add_to_hash_dict(mediabyte_str):
    """Add Yota/Cue/Sample/bit string to hash_dict JSON."""

    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)
    
    #myObj = check_ono(mediabyte_str)
    

    def hash(mediabyte_str):
        m = hashlib.sha256()
        m.update(mediabyte_str.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)

    no_whitespace_mediabyte_str = mediabyte_str.replace(' ', '_')
    calculated_hash = hash(no_whitespace_mediabyte_str)

    # m = hashlib.sha256()
    # m.update(mediabyte_str.encode())
    # calculated_hash = m.hexdigest()[:11]
    
    # check for Mixtape object
    #mixtape_check = re.search('\.y\.', mediabyte_str)
    
    #if not mixtape_check:
    hash_dict[calculated_hash] = mediabyte_str.replace(' ','_')
    #else:
    #    hash_dict[calculated_hash] = myObj.omm_oneline()
    
    with open(hash_dict_filename, 'w') as f:
        json.dump(hash_dict, f)
    
        

def check_ono(ono_str, string_arg=False):
    with open(hash_dict_filename, 'r') as f:
        hash_dict = json.load(f)

    keys = list(hash_dict.keys())
    m = re.search('^o\.([a-zA-Z0-9]+)', ono_str)
    o_hash = m.group(1)
    results = []
    for key in keys:
        if o_hash == key[:len(o_hash)]:
            #print(key)
            if not string_arg:
                myObject = lib.Convert.omm(hash_dict[key])
                #return myObject
                results.append(myObject)
            else:
                #return hash_dict[key]
                results.append(hash_dict[key])
    if len(results) > 1:
        return "Multiple matches, please add characters and try again"
    elif len(results) == 1:
        return results[0]


hash_dict = load_hash_dict()



from . import srt_keyword_parser as srt
import subprocess, glob, os
from copy import deepcopy
from . import cnf
from . import dl2



def srt_search(self, omm, Mixtape, keyword, clip_length=10, do_not_combine_overlapping=False):

    #def check_cache(youtube_hash):
    # check if .srt is already downloaded, use cache version

    #     youtube_url = 'https://youtube.com/watch?v=' + youtube_hash        
    #     srt_filename = cnf.package_path + cnf.os_sep + 'srt' + cnf.os_sep + '*.srt'
    #     srt_files = glob.glob(srt_filename)

    #     for srt_filename in srt_files:
    #         if self.youtube_hash in srt_filename:
    #             return srt_filename
        
    # cache_result = check_cache(self.youtube_hash)
    # # use the cache subtitles if present
    # try:
    #     #cache_result is not None
    #     latest_file = cache_result
    # # or download if not
    # except:
    #     srt_filename = dl2.dl_srt(youtube_url)
    #     latest_file = srt_filename
    
    youtube_url = 'https://youtube.com/watch?v=' + self.youtube_hash
    srt_filename = dl2.dl_srt(youtube_url)
    result = srt.main(srt_filename, keyword)
    
    if len(result) > 1:
        myYota = omm(result[0])
        if clip_length:
            myYota = myYota.to_sample(omm, add=clip_length)
        myYota.title = self.title + ' keyword ' + keyword + ' 1'
        myYota.update()
        myMixtape = Mixtape(myYota)
        # print('startet Mixtape with object:', result[0])
        for i, item in enumerate(result[1:]):
            myYota = omm(item)
            if clip_length:
                myYota = myYota.to_sample(omm, add=clip_length)
            myYota.title = self.title + ' keyword ' + keyword + ' ' + str(i+2)
            myYota.update()
            myMixtape += myYota
            # print('added item:', item)
    
    elif len(result) == 1:
        myYota = omm(result[0])
        if clip_length:
            myYota = myYota.to_sample(omm, add=clip_length)
        myYota.title = self.title + ' keyword ' + keyword + ' 1'
        myYota.update()
        myMixtape = Mixtape(myYota)

    elif len(result) == 0:
        myMixtape = None


    def combine_overlapping(inMix):
        result = []
        skip_next = False
        for i, item in enumerate(inMix[:-1]):
            if skip_next == True:
                skip_next = False
                continue
            test = item.time_end > inMix[i+1].time_start
            test2 = item.youtube_hash == inMix[i+1].youtube_hash
            if test and test2:
                #print(item.time_end, inMix[i+1].time_start)
                newSample = deepcopy(item)
                newSample.time_end = inMix[i+1].time_end
                newSample.update()
                result.append(newSample)
                skip_next = True
            else:
                result.append(item)
        
        if skip_next == False:
            result.append(inMix[-1])
        myMix = Mixtape(result[0])
        uniques = []
        for item in result:
            if item not in uniques:
                uniques.append(item)
        # NB: Hacky solution, filtering doubled results (coming from where?)
        for item in uniques[1:]:
            myMix += item
        return(myMix)
    

    if not do_not_combine_overlapping:
        if isinstance(myMixtape, Mixtape) and len(myMixtape.content) > 1:
            # NB: Hacky, overconfident in its own success
            while True:
                if len(myMixtape.content) > 1:
                    myNewMixtape = combine_overlapping(myMixtape)
                    if myNewMixtape.omm() == myMixtape.omm():
                        return(myNewMixtape)
                    else:
                        myMixtape = myNewMixtape
                else:
                    return(myMixtape)

    return(myMixtape)



"""yNo Sample String Parser (Yota implementation)."""
import re


"""
                yNo Sample String Parser 2.2
                       March 5th 2019
                        by d@v1d.dk
What's New: Fix 11-character tag bug
"""


def get_time_codes(tag_str):

    """Extract YouTube time codes from CLI string."""

    mylist = tag_str.split(".")
   ##print('mylist in get_time_codes():', str(mylist))
    mytimes = []
    newtimes = []
    for item in mylist:

        yt_time_match = re.search(r'^(\d+[hms]){1,3}$', item)
        if yt_time_match:
            mytimes.append(yt_time_match.group(0))
        else:
            newtimes.append(item)
   ##print('mytimes:', mytimes, 'newtimes: ', ".".join(newtimes))
    return(mytimes, ".".join(newtimes))


def get_titles(tag_str):

    """Extract titles from CLI string."""

    #print('tag_str:', tag_str)
    mylist = tag_str.split(".")
   ##print('tag_str at start of get_titles: ', tag_str)
    #print('mylist at start of get_titles: ', mylist)
    mytitles = []
    newtitles = []
    for i, item in enumerate(mylist):
        title_match = re.search('^[A-Z]', item)  # match on uppercase first character
        if title_match:
            #if len(item) != 11: #NB - very hacky solution, uses string length
            mytitles.append(item)
            for item in mylist[i+1:]:
                newtitles.append(item)
                #print('item:', item)
                #print('mytitles:', mytitles)
                #print('newtitles:', newtitles)
            return(mytitles, ".".join(newtitles))
        else:
            newtitles.append(item)
        # else:
        #     newtitles.append(item)
    return(mytitles, ".".join(newtitles))


def get_youtube_url(tag_str):

    """Extract YouTube URL or hash from string."""

    mylist = tag_str.split(".")

    newtimes = []
    result = ""


    for item in mylist:

        http_match = re.search('^http', item)  # match on starts with http
        yt_url_match = re.search('^youtube.com/watch?v=([a-zA-Z0-9-]{11})', item)
        # match on starts with 'youtube.com/'
        yt_url_match_2 = re.search('^youtu.be/', item)     # match on starts with 'youtu.be/'
        yt_hash_match = re.search('^[a-zA-Z0-9\-\_]{11}$', item)  # match on YouTube hash format

        if http_match:

            result = item
        elif yt_url_match:
            youtube_hash = item
            result = item
        elif yt_url_match_2:

            result = item
        elif yt_hash_match:
            youtube_hash = yt_hash_match.group(0)
            result = youtube_hash
            # NB: Possibly hacky, not sure
            if tag_str[:2] == 'y.':
                newtimes.append(tag_str[13:])
            else:
                newtimes.append(tag_str[11:]) 
            
            return(result, ".".join(newtimes[1:])[1:])

        else:
            newtimes.append(item)
    if not result:
        raise ValueError("No YouTube hash match found")

    return(result, ".".join(newtimes)[1:])


def get_bit_object(input_str):

    m = re.search('\.b\.', input_str)
    if m:
        bit_object_results = input_str.split('.b.')
    else:
        bit_object_results = [input_str]
    new_objects = []
    for item in bit_object_results:
        new_objects.append(item)

    if len(new_objects) > 1:
        leading_str = new_objects[0]
    else:
        return([], input_str)

    result = ""
    result_list = []
    
    for item in bit_object_results[1:]:

        result = 'b.' + item
        result_list.append(result)

    return(result_list, leading_str)


def parse_element_types(str_input):
    """Parse string for time codes, titles and URL."""
    bit_object_list, rest0 = get_bit_object(str_input)
   ##print('rest0 after get_bit_objects:', rest0)
   ##print('bit_object_list:', bit_object_list)
    yt_url, rest1 = get_youtube_url(rest0)
   ##print('rest1 after get_youtube_url:', rest1)
    time_codes, rest2 = get_time_codes(rest1)
   ##print('rest2 after get_time_codes:', rest2)
    titles, rest3 = get_titles(rest2)
   ##print('rest3 after get_titles:', rest3)
    
    if rest3:
        tags = rest3.split(".")
    else:
        tags = []
   ##print('tags after split: ',tags)
    #print('rest3:', rest3)

    return(time_codes, titles, yt_url, tags, bit_object_list)


def print_report(result_in):
    """Print data fields."""
    
    if result_in[0]:
       print("Time Codes: " + " ".join(result_in[0][:]))
    if result_in[1]:
       print("Titles: " + " ".join(result_in[1]))
    if result_in[2]:
       print("YouTube URL: " + result_in[2])
    if result_in[3]:
       print("Tags: " + " ".join(result_in[3]))



def main(yno_string):
    """Main function, takes a yNo format string, returns string result."""
    result = parse_element_types(yno_string)
    return result


# def build_yota_object(result_list):
#     """Builds and returns Yota object."""
#     times_list = result_list[0]
#     yturl_str = result_list[2]
#     tags_list = result_list[3]

#     if len(times_list) == 2:

#         timesinsec_list = []
#         for time in times_list:
#             seconds_match = re.search(r'(\d{1,2})s', time)
#             if seconds_match:
#                 secs = int(seconds_match.group(1))
#             else:
#                 secs = 0
#             minutes_match = re.search(r'(\d{1,2})m', time)
#             if minutes_match:
#                 minutes = int(minutes_match.group(1))
#             else: minutes = 0
#             hour_match = re.search(r'(\d{1,2})h', time)
#             if hour_match:
#                 hours = int(hour_match.group(1))
#             else: hours = 0

#             total_secs = secs + 60*minutes + 3600*hours

#             timesinsec_list.append(total_secs)




#         try:
#             new_sample = Sample(url=yturl_str, title=" ".join(result_list[1]), \
#             time_start=timesinsec_list[0], time_end=timesinsec_list[1], tags=tags_list)
#             return new_sample
#         except:
#             raise ValueError("Exactly two time codes needed")
    
    
#     elif len(times_list) == 1:

#         timesinsec_list = []
#         for time in times_list:
#             seconds_match = re.search(r'(\d{1,2})s', time)
#             if seconds_match:
#                 secs = int(seconds_match.group(1))
#             else:
#                 secs = 0
#             minutes_match = re.search(r'(\d{1,2})m', time)
#             if minutes_match:
#                 minutes = int(minutes_match.group(1))
#             else: minutes = 0
#             hour_match = re.search(r'(\d{1,2})h', time)
#             if hour_match:
#                 hours = int(hour_match.group(1))
#             else: hours = 0

#             total_secs = secs + 60*minutes + 3600*hours

#             timesinsec_list.append(total_secs)




#         try:
#             new_cue = Cue(yt_hash=result_list[2], title=" ".join(result_list[1]), \
#             time_start=timesinsec_list[0][0], tags=tags_list)
#             return new_cue
#         except:
#             raise ValueError("Exactly one time codes needed")
        



#!/home/dd/anaconda3/bin/python

import unittest, os
from files import lib
from files import bit
from files import yota_fuzz
from files import omm_file_parser


class Testomm(unittest.TestCase):

    def setUp(self):
        pass


### TAGS AND TITLE

    def test_sample_full(self):
        sample_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors.podcast.intro.1m58s'
        result = lib.Convert.omm(sample_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.time_start, 1)
        self.assertEqual(result.time_end, 118)
        self.assertEqual(result.time(), '1m57s')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_cue_full(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors.podcast.intro'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.time_start, 1)
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_yota_full(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.Welcome_&_Sponsors.podcast.intro'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')


#### NO TAGS
    
    def test_sample_no_tags(self):
        sample_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors.1m58s'
        result = lib.Convert.omm(sample_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.time_start, 1)
        self.assertEqual(result.time_end, 118)
        self.assertEqual(result.time(), '1m57s')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_cue_no_tags(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.time_start, 1)
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_yota_no_tags(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.Welcome_&_Sponsors'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__()[:7], 'Welcome')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')


### NO TITLE

    def test_sample_no_title(self):
        sample_test_str = 'y.8ZtHhF9Lt_8.1s.podcast.intro.1m58s'
        result = lib.Convert.omm(sample_test_str)
        self.assertEqual(result.__repr__()[:8], 'MySample')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.time_start, 1)
        self.assertEqual(result.time_end, 118)
        self.assertEqual(result.time(), '1m57s')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_cue_no_title(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.1s.podcast.intro'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__()[:5], 'MyCue')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_yota_no_title(self):
        cue_test_str = 'y.8ZtHhF9Lt_8.podcast.intro'
        result = lib.Convert.omm(cue_test_str)
        self.assertEqual(result.__repr__(), 'MyYota  (podcast, intro)                                     o.fd3faf525a1')
        self.assertEqual(result.tags, ['podcast','intro'])
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')


### NO TAGS & TITLE

    def test_sample_no_title_or_tags(self):
        sample_test_str = 'y.8ZtHhF9Lt_8.1s.1m58s'
        result = lib.Convert.omm(sample_test_str)
        self.assertEqual(result.__repr__(), 'MySample  1m57s                                              o.efed709609f')
        my_str_name = result.__str__()
        self.assertEqual(my_str_name[-5:], '1m58s')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_cue_no_title_or_tags(self):
        sample_test_str = 'y.8ZtHhF9Lt_8.1m58s'
        result = lib.Convert.omm(sample_test_str)
        my_str_name = result.__str__()
        self.assertEqual(my_str_name[-5:], '1m58s')
        self.assertEqual(result.time_start, 118)
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')

    def test_yota_no_title_or_tags(self):
        sample_test_str = 'y.8ZtHhF9Lt_8'
        result = lib.Convert.omm(sample_test_str)
        my_str_name = result.__str__()
        self.assertEqual(my_str_name, 'y.8ZtHhF9Lt_8')
        self.assertEqual(result.youtube_hash, '8ZtHhF9Lt_8')


### MIXTAPE

    def test_mixtape_full(self):
        
        mixtape_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors.podcast.intro.1m58s..y.8ZtHhF9Lt_8.1m59s.Intro_banter.podcast.episode.overview.3m46s..y.8ZtHhF9Lt_8.3m47s.Overview.episode.overview.6m16s..y.8ZtHhF9Lt_8.6m17s.Sponsor:_ITPro_TV.sponsor.10m13s'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.__repr__()[:17], 'mediabyte.Mixtape')
        self.assertEqual(len(mixtapeObj.content), 4)
        self.assertEqual(mixtapeObj[0].tags, ['podcast','intro'])
        self.assertEqual(mixtapeObj[0].time_end, 118)
        self.assertEqual(mixtapeObj[0].youtube_hash, '8ZtHhF9Lt_8')

    def test_mixtape_no_tags(self):
        
        mixtape_test_str = 'y.8ZtHhF9Lt_8.1s.Welcome_&_Sponsors.1m58s..y.8ZtHhF9Lt_8.1m59s.Intro_banter.podcast.episode.overview.3m46s..y.8ZtHhF9Lt_8.3m47s.Overview.episode.overview.6m16s..y.8ZtHhF9Lt_8.6m17s.Sponsor:_ITPro_TV.10m13s'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.__repr__()[:17], 'mediabyte.Mixtape')
        self.assertEqual(len(mixtapeObj.content), 4)
        self.assertEqual(mixtapeObj[0].time_end, 118)
        self.assertEqual(mixtapeObj[0].youtube_hash, '8ZtHhF9Lt_8')

    def test_mixtape_no_title(self):
        
        mixtape_test_str = 'y.8ZtHhF9Lt_8.1s.podcast.intro.1m58s..y.8ZtHhF9Lt_8.1m59s.podcast.episode.overview.3m46s..y.8ZtHhF9Lt_8.3m47s.episode.overview.6m16s..y.8ZtHhF9Lt_8.6m17s.sponsor.10m13s'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.__repr__()[:17], 'mediabyte.Mixtape')
        self.assertEqual(len(mixtapeObj.content), 4)
        self.assertEqual(mixtapeObj[0].tags, ['podcast','intro'])
        self.assertEqual(mixtapeObj[0].time_end, 118)
        self.assertEqual(mixtapeObj[0].youtube_hash, '8ZtHhF9Lt_8')

    def test_mixtape_no_tags_or_title(self):
        
        mixtape_test_str = 'y.8ZtHhF9Lt_8.1s.1m58s..y.8ZtHhF9Lt_8.1m59s.3m46s..y.8ZtHhF9Lt_8.3m47s.6m16s..y.8ZtHhF9Lt_8.6m17s.10m13s'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.__repr__()[:17], 'mediabyte.Mixtape')
        self.assertEqual(len(mixtapeObj.content), 4)
        self.assertEqual(mixtapeObj[0].time_end, 118)
        self.assertEqual(mixtapeObj[0].youtube_hash, '8ZtHhF9Lt_8')

    def test_mixtape_no_tags_or_title_or_time_codes(self):
        
        mixtape_test_str = 'y.8ZtHhF9Lt_8..y.8ZtHhF9Lt_8..y.8ZtHhF9Lt_8.3m47s.6m16s..y.8ZtHhF9Lt_8.6m17s.10m13s'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.__repr__()[:17], 'mediabyte.Mixtape')
        self.assertEqual(len(mixtapeObj.content), 4)
        self.assertEqual(mixtapeObj[0].youtube_hash, '8ZtHhF9Lt_8')

    def test_mixtape_oneline_format(self):
        mixtape_test_str = 'y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m..y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m'
        mixtapeObj = lib.Convert.omm(mixtape_test_str)
        self.assertEqual(mixtapeObj.omm_oneline(), 'y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m..y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m')


## YOUTUBE URL PARSING
    
    # no time code, only YouTube hash
    # def test_youtube_url_parsing_0(self): 
    #     youtube_url_fourth_format = 'NI1L8ZJgA9o'
    #     yota_object = lib.Convert.omm(youtube_url_fourth_format)
    #     self.assertEqual(yota_object.omm, 'y.NI1L8ZJgA9o')

    # no time code, basic YouTube URL
    def test_youtube_url_parsing(self): 
        youtube_url_first_format = 'https://www.youtube.com/watch?v=NI1L8ZJgA9o'
        yota_object = lib.Convert.omm(youtube_url_first_format)
        self.assertEqual(yota_object.omm, 'y.NI1L8ZJgA9o')

    # with time code, from YouTube share link with time code (old format)
    def test_youtube_url_parsing_2(self):
        youtube_url_second_format = 'https://youtu.be/NI1L8ZJgA9o?t=60s'
        cue_object = lib.Convert.omm(youtube_url_second_format)
        self.assertEqual(cue_object.omm, 'y.NI1L8ZJgA9o.1m')

    # with time code, from YouTube share link with time code (new format)
    def test_youtube_url_parsing_4(self):
        youtube_url_second_format = 'https://youtu.be/NI1L8ZJgA9o?t=120'
        cue_object = lib.Convert.omm(youtube_url_second_format)
        self.assertEqual(cue_object.omm, 'y.NI1L8ZJgA9o.2m')

    # with time code, from clicking YouTube logo when in embedded player
    def test_youtube_url_parsing_3(self):
        youtube_url_third_format = 'https://www.youtube.com/watch?time_continue=60&v=drRQVI58c-E'
        cue_object = lib.Convert.omm(youtube_url_third_format)
        self.assertEqual(cue_object.omm, 'y.drRQVI58c-E.1m')


## JAVASCRIPT HTML PLAYER Mixtape.player() 

    def test_mixtape_player(self):
        test_mix_object = lib.Convert.omm('y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m..y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m')
        obj_jule = test_mix_object.iframe()
        test_jule = '<p>\nmediabyte.Mixtape <strong>o.e994f78f2cf</strong> Koreans <strong>o.aa13e150023</strong> Koreans <strong>o.aa13e150023</strong> </p>  <iframe width="360" height="216" title="Koreans on Metoo" src="https://www.youtube.com/embed/drRQVI58c-E?start=78&end=120&rel=0&autoplay=0&enablejsapi=1" frameborder=0 allowfullscreen></iframe> <iframe width="360" height="216" title="Koreans on Metoo" src="https://www.youtube.com/embed/drRQVI58c-E?start=78&end=120&rel=0&autoplay=0&enablejsapi=1" frameborder=0 allowfullscreen></iframe> <!DOCTYPE html>\n                                <html lang="en">\n                                <head>\n                                <meta charset="utf-8">\n                                <meta name="viewport" content="width=device-width, initial-scale=1">\n                                <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css">\n                                <script src="https://ajax.googleapis.com/ajax/libs'

        self.assertEqual(obj_jule[:1000], test_jule[:1000])


## IFRAME METHODS

    def test_cue_iframe(self):
        test_cue_object = lib.Convert.omm('y.gCLoXNL-AAI.1s.Rocket Beans Live')  # live stream, does it matter?
        test_iframe = '<center><iframe width="360" height="216"  src="https://www.youtube.com/embed/gCLoXNL-AAI?start=1&rel=0&autoplay=1" frameborder=0 allowfullscreen></iframe></center>'

        self.assertEqual(test_cue_object.iframe(), test_iframe)

    def test_yota_iframe(self):
        test_cue_object = lib.Convert.omm('y.gCLoXNL-AAI.Rocket Beans Live')  # live stream, does it matter?
        test_iframe = '<center><iframe width="360" height="216"  src="https://www.youtube.com/embed/gCLoXNL-AAI?start=0&rel=0&autoplay=1" frameborder=0 allowfullscreen></iframe></center>'

        self.assertEqual(test_cue_object.iframe(), test_iframe)

    def test_sample_iframe(self):
        test_cue_object = lib.Convert.omm('y.drRQVI58c-E.1s.Rocket Beans Live.4s')  # live stream, does it matter?
        test_iframe = '<center><iframe width="360" height="216"  src="https://www.youtube.com/embed/drRQVI58c-E?start=1&end=4&rel=0&autoplay=1" frameborder=0 allowfullscreen></iframe></center>'

        self.assertEqual(test_cue_object.iframe(), test_iframe)


# MANUAL ADDITION OF ATTRIBUTES

    def test_yota_manual_attributes(self):

        yota_object = lib.Convert.omm('y.drRQVI58c-E')
        yota_object.tags = ['interview','asia']
        yota_object.title = 'Koreans on Metoo'
        yota_object.update()
        self.assertEqual(yota_object.omm, 'y.drRQVI58c-E.Koreans_on_Metoo.interview.asia')

    def test_cue_manual_attributes(self):

        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m')
        yota_object.tags = ['interview','asia']
        yota_object.title = 'Koreans on Metoo'
        yota_object.update()
        self.assertEqual(yota_object.omm, 'y.drRQVI58c-E.5m.Koreans_on_Metoo.interview.asia')

    def test_sample_manual_attributes(self):

        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m.5m10s')
        yota_object.tags = ['interview','asia']
        yota_object.title = 'Koreans on Metoo'
        yota_object.update()
        self.assertEqual(yota_object.omm, 'y.drRQVI58c-E.5m.Koreans_on_Metoo.interview.asia.5m10s')
    

# YOTA TO CUE TO SAMPLE TO MIXTAPE TRAVERSAL
    
    def test_yota_to_cue_to_sample(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E')
        yota_object.tags = ['interview','asia']
        yota_object.title = 'Koreans on Metoo'
        yota_object.update()
        yota_object.omm += '.1m18s' # add time_start
        new = lib.Convert.omm(yota_object.omm)  # parse new Cue object
        test_str = 'y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia'
        self.assertEqual(new.omm, test_str) # test for cue

    def test_cue_to_sample(self):
        test_cue = lib.Convert.omm('y.drRQVI58c-E.Koreans_on_Metoo.interview.asia.1m18s')
        test_cue.omm += '.2m'
        newer = lib.Convert.omm(test_cue.omm)
        test_str = 'y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m'
        self.assertEqual(newer.omm, test_str) # test for sample

    def test_sample_to_mixtape(self):
        test_sample = lib.Convert.omm('y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m')
        mixtape = test_sample.omm + '..' + test_sample.omm
        mix_obj = lib.Convert.omm(mixtape)
        test_str = 'y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m..y.drRQVI58c-E.1m18s.Koreans_on_Metoo.interview.asia.2m'
        self.assertEqual(mix_obj.omm_oneline(), test_str)


# MIXTAPE WITH ONLY YOTA AND CUE

    def test_mixtape_with_only_yotas(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.NI1L8ZJgA9o')
        self.assertEqual(yota_object[1].omm, 'y.NI1L8ZJgA9o')

    def test_mixtape_with_only_cues(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m..y.NI1L8ZJgA9o.5m')
        self.assertEqual(yota_object[1].omm, 'y.NI1L8ZJgA9o.5m')

    def test_mixtape_with_cue_and_yota(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.NI1L8ZJgA9o.5m')
        self.assertEqual(yota_object[1].omm, 'y.NI1L8ZJgA9o.5m')

    def test_mixtape_with_yota_and_cue(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m..y.NI1L8ZJgA9o')
        self.assertEqual(yota_object[1].omm, 'y.NI1L8ZJgA9o')


# MIXTAPE FROM FILE

    def test_mixtape_from_file_sample(self):
        yota_object = lib.Convert.omm('mixtapes/hackers2018.omm')
        self.assertEqual(yota_object[3].title, 'John Oliver: Trade')
    
    def test_mixtape_from_file_length(self):
        yota_object = lib.Convert.omm('mixtapes/hackers2018.omm')
        self.assertEqual(len(yota_object.content), 4)


# YOTA ARITHMETIC

    def test_yota_plus_yota_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o')

    def test_cue_plus_cue_equals_mixtape(self):
        cue_object = lib.Convert.omm('y.drRQVI58c-E.5m')
        cue_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.7m')
        mixtape_object = cue_object + cue_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.7m')

    def test_sample_plus_sample_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m.6m')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.7m.8m')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.7m.8m')


    def test_yota_plus_cue_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.7m')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.7m')

    def test_cue_plus_yota_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.7m')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o')

    def test_cue_plus_sample_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.7m.8m')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.7m.8m')

    def test_sample_plus_cue_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m.6m3s')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.7m')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.7m')

    def test_sample_plus_yota_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E.5m.6m3s')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o')

    def test_yota_plus_sample_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.4m10s.4m30s')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[1].omm, 'y.NI1L8ZJgA9o.4m10s.4m30s')


    def test_mixtape_plus_yota_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[2].omm, 'y.NI1L8ZJgA9o')

    def test_mixtape_plus_cue_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.4m10s')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[2].omm, 'y.NI1L8ZJgA9o.4m10s')
    
    def test_mixtape_plus_sample_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.NI1L8ZJgA9o.4m10s.4m30s')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[2].omm, 'y.NI1L8ZJgA9o.4m10s.4m30s')

    def test_mixtape_plus_mixtape_equals_mixtape(self):
        yota_object = lib.Convert.omm('y.drRQVI58c-E..y.drRQVI58c-E')
        yota_object2 = lib.Convert.omm('y.drRQVI58c-E..y.drRQVI58c-E')
        mixtape_object = yota_object + yota_object2
        self.assertEqual(mixtape_object[3].omm, 'y.drRQVI58c-E')

    def test_mixtape_slicing(self):
        yota_str = 'y.NI1L8ZJgA9o.4m10s.First Clip.4m30s..y.NI1L8ZJgA9o.5m10s.Second Clip.5m30s..y.NI1L8ZJgA9o.6m10s.Third Clip.6m30s'
        yota_object = lib.Convert.omm(yota_str)
        new_object = yota_object[1:]
        self.assertEqual(len(new_object.content), 2)

    def test_sample_indexing(self):
        yota_object = lib.Convert.omm('y.NI1L8ZJgA9o.10s.30s')
        new_object = yota_object[10]
        self.assertEqual(new_object.time_start, 20)

    
## PADDING AND SHIFTING

    def test_cue_shift_forwards(self):
        yota_object = lib.Convert.omm('y.youtubehash.5s')
        new_object = yota_object.shift(5)
        self.assertEqual(new_object.omm, 'y.youtubehash.10s')

    def test_cue_shift_backwards(self):
        yota_object = lib.Convert.omm('y.youtubehash.10s')
        new_object = yota_object.shift(-5)
        self.assertEqual(new_object.omm, 'y.youtubehash.5s')

    def test_sample_pad_front(self):
        yota_object = lib.Convert.omm('y.youtubehash.5s.10s')
        new_object = yota_object.pad(1)
        self.assertEqual(new_object.omm, 'y.youtubehash.4s.10s')

    def test_sample_pad_back(self):
        yota_object = lib.Convert.omm('y.youtubehash.5s.10s')
        new_object = yota_object.pad(-1)
        self.assertEqual(new_object.omm, 'y.youtubehash.5s.11s')


## YOTA SRT SEARCH

    def test_srt_search(self):
        url = 'https://www.youtube.com/watch?v=Utu0RNjf_h8'
        myYota = lib.Convert.omm(url)
        mentions = myYota.srt_search('kitchen')
        self.assertEqual(len(mentions), 2)
    # def test_srt_search_2(self):
    #     url = 'https://www.youtube.com/watch?v=Utu0RNjf_h8'
    #     myYota = lib.Convert.omm(url)
    #     mentions = myYota.srt_search('kitchen')
    #     self.assertEqual(mentions[1].omm, 'y.Utu0RNjf_h8.6m23s_keyword_kitchen_2.6m33s')


## BIT OBJECT PARSING

    def test_yota_with_sample_bit(self):
        YotaWithSampleBit = 'y._QYngRrbsKo.My_YouTube_Clip.b.2TKY42w.1s.SecNow.mp3.security.podcast.25s'
        self.assertEqual( lib.Convert.omm(YotaWithSampleBit).bits[0].omm, 'b.2TKY42w.1s.SecNow.mp3.security.podcast.25s')
        self.assertEqual( lib.Convert.omm(YotaWithSampleBit).bits[0].title, 'SecNow')
    def test_yota_with_tag_and_sample_bit(self):
        YotaWithTagAndSampleBit = 'y._QYngRrbsKo.My_YouTube_Clip.video.b.2TKY42w.Security_Now.mp3.security.podcast.1s.25s'
        self.assertEqual( lib.Convert.omm(YotaWithTagAndSampleBit).title, 'My YouTube Clip')
        self.assertEqual( lib.Convert.omm(YotaWithTagAndSampleBit).bits[0].title, 'Security Now')
    def test_yota_with_bit(self):
        YotaWithBit = 'y._QYngRrbsKo.My_YouTube_Clip.video.b.2TKY42w.Security_Now.security.podcast'
        self.assertEqual( lib.Convert.omm(YotaWithBit).tags, ['video'])
    def test_yota_with_tags(self):
        YotaWithTags = 'y._QYngRrbsKo.My_YouTube_Clip.tags.video'
        self.assertEqual( lib.Convert.omm(YotaWithTags).tags, ['tags','video'])
    def test_cue_with_tags(self):
        CueWithTags = 'y._QYngRrbsKo.My_YouTube_Clip.tags.video.1m'
        self.assertEqual( lib.Convert.omm(CueWithTags).url, 'https://www.youtube.com/embed/_QYngRrbsKo?start=60&rel=0&autoplay=1')
    
    def test_sample_with_tags(self):
        SampleWithTags = 'y._QYngRrbsKo.My_YouTube_Clip.tags.video.1m.2m'
        self.assertEqual( lib.Convert.omm(SampleWithTags).url, 'https://www.youtube.com/embed/_QYngRrbsKo?start=60&end=120&rel=0&autoplay=1')


## BIT OBJECTS IN __REPR__
    
    def test_sample_with_bits_repr(self):

        mySampleWithBits = lib.Convert.omm('y._QYngRrbsKo.My_YouTube_Clip.video.1m1s.1m10s.b.2TKY42w.Sec_Now.security.podcast.b.2TKY42w.Sec_Now.mp3.security.podcast.6m1s.7m25s.b.2TKY42w.Sec_Now.mp3.security.podcast.8m1s.9m25s')
        self.assertEqual(mySampleWithBits.__repr__(), 'My YouTube Clip  (video)  9s  [Sec Now, Sec Now, Sec Now]                                 o.c856d3a2076')

    def test_cue_with_bits_repr(self):
        myCueWithBits = lib.Convert.omm('y._QYngRrbsKo.My_YouTube_Clip.1m1s.b.2TKY42w.Sec_Now.security.podcast.b.2TKY42w.Sec_Now.mp3.security.podcast.6m1s.7m25s.b.2TKY42w.Sec_Now.mp3.security.podcast.8m1s.9m25s')
        self.assertEqual(myCueWithBits.__repr__(), 'My YouTube Clip  [Sec Now, Sec Now, Sec Now]                                              o.f8babaddfe2')

    def test_yota_with_bits_second_bit_omm(self):
        myYotaWithBits = lib.Convert.omm('y._QYngRrbsKo.My_YouTube_Clip.b.2TKY42w.Sec_Now.security.podcast.b.2TKY42w.Sec_Now.mp3.security.podcast.6m1s.7m25s.b.2TKY42w.Sec_Now.mp3.security.podcast.8m1s.9m25s')
        self.assertEqual(myYotaWithBits.__repr__(), 'My YouTube Clip  [Sec Now, Sec Now, Sec Now]                                              o.c2c219421b5')


## BIT ONLY PARSING

    # Bit.Link

    def test_bit_with_no_title_or_tags(self):
        bit_str = 'b.2NYwBFd'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.bitly_hash, '2NYwBFd')

    def test_bit_with_no_title(self):
        bit_str = 'b.2NYwBFd.some.tags'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.__repr__(), 'MyBit  (some, tags)')

    def test_bit_with_title_and_tags(self):
        bit_str = 'b.2NYwBFd.MyTitle.some.tags'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.omm, 'b.2NYwBFd.MyTitle.some.tags')

    def test_bit_with_title(self):
        bit_str = 'b.2NYwBFd.MyTitle'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.omm, 'b.2NYwBFd.MyTitle')

    def test_bit_custom_link(self):
        bit_str = 'b.sn-704'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.link, 'http://twit.cachefly.net/audio/sn/sn0704/sn0704.mp3')


    # Bit.Mp3

    def test_bit_mp3_cue_no_title(self):
        bit_str = 'b.sn-704.5m.mp3'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.time_start, 300)

    def test_bit_mp3_sample_no_title(self):
        bit_str = 'b.sn-704.5m.5m59s.mp3'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.time(), '59s')

    def test_bit_mp3_type(self):
        bit_str = 'b.sn-704.mp3'
        myBit = lib.Convert.omm(bit_str)
        if isinstance(myBit, bit.Mp3):
            checked_out = True
        self.assertEqual(checked_out, True)

    def test_bit_mp3_sample_with_title(self):
        bit_str = 'b.sn-704.Security Now Quote.mp3.5m.2h'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.title, 'Security Now Quote')

    def test_bit_mp3_cue_with_title(self):
        bit_str = 'b.sn-704.Security Now Quote.mp3.5m'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.title, 'Security Now Quote')

    def test_bit_mp3_yota_with_title(self):
        bit_str = 'b.sn-704.Security Now Quote.mp3'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.title, 'Security Now Quote')

    def test_bit_mp3_repr(self):
        bit_str = 'b.sn-704.Security Now Quote.mp3'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.__repr__(), 'Security Now Quote  (mp3)')

    def test_bit_repr(self):
        bit_str = 'b.sn-704.Security Now Quote'
        myBit = lib.Convert.omm(bit_str)
        self.assertEqual(myBit.__repr__(), 'Security Now Quote')

##  Sample Subtraction

    def test_sample_subtraction(self):
        mySampleStr = 'y.Ufr7O0qtX9U.1s.1h30m'
        myBigSample = lib.Convert.omm(mySampleStr)
        mySmallSampleStr = 'y.Ufr7O0qtX9U.1h1m.1h2m'
        mySmallSample = lib.Convert.omm(mySmallSampleStr)
        firstBite, secondBite = myBigSample - mySmallSample
        self.assertEqual(firstBite.time(), '1h59s')
        self.assertEqual(secondBite.time(), '28m')


##  YOTA METHODS

    def test_yota_methods(self):
        myYotaStr = 'y.Ufr7O0qtX9U'
        myYota = lib.Convert.omm(myYotaStr)
        self.assertEqual(myYota.methods(), ['hash','iframe','methods','play','srt_search','update','vlc'])

    def test_cue_methods(self):
        myCueStr = 'y.Ufr7O0qtX9U.1s'
        myCue = lib.Convert.omm(myCueStr)
        self.assertEqual(myCue.methods(), ['hash','iframe','methods','play','srt_search','update','vlc'])

    def test_sample_methods(self):
        mySampleStr = 'y.Ufr7O0qtX9U.1s.10s'
        mySample = lib.Convert.omm(mySampleStr)
        self.assertEqual(mySample.methods()[:6], ['beta_format', 'beta_format_2', 'bits', 'first_name', 'format', 'hash'])

    def test_mixtape_methods(self):
        myCueStr = 'y.Ufr7O0qtX9U.1s.10s..y.Ufr7O0qtX9U.1s.10s'
        myCue = lib.Convert.omm(myCueStr)
        #print(myCue.methods()[:5])
        self.assertEqual(myCue.methods()[:5], ['add_tags', 'append', 'browser_player', 'content', 'find'])

    def test_bit_methods(self):
        myBitStr = 'b.sn-704.MyBit'
        myBit = lib.Convert.omm(myBitStr)
        self.assertEqual(myBit.methods(), ['hash','iframe', 'methods', 'play', 'update', 'vlc'])

    def test_bit_mp3_methods(self):
        myBitStr = 'b.sn-704.MyBit.mp3'
        myBit = lib.Convert.omm(myBitStr)
        self.assertEqual(myBit.methods(), ['hash','iframe', 'methods', 'play', 'time', 'update', 'vlc'])





if __name__ == '__main__':
    unittest.main()    



import json
from . import cnf

class MediabyteHashObj(dict):
    
    def __init__(self):
        filename = cnf.hash_dict_path
        with open(filename, 'r') as f:
            hash_dict = json.load(f)
        for key, value in hash_dict.items():
            self[key] = value


    def __getattribute__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(key)


    def __setattr__(self, key, value):
        self[key] = value
    
    
    


#!/home/dd/anaconda3/bin/python
#      srt_cli.py v0.2 February 26th 2019
# - get links to keyword occurrences in YouTube 
#   video from the auto-generated subtitles
# - demonstration of interacting with
#   the yota module in practice

from yota import yNo
import sys

## SETUP ## 
# ARGUMENT HANDLING
try: # check for mandatory youtube URL argument
    youtube_url_from_arg = sys.argv[1]
except:
    exit('Error: no youtube URL argument found')
try: # check for mandatory search term argument
    search_term_from_arg = sys.argv[2]
except:
    exit('Error: no search term argument found')
try: # check if optional clip length argument was passed
    clip_length_from_arg = int(sys.argv[3])
except:
    # or default to clip length 10 seconds
    clip_length_from_arg = 10

## MAIN ##
# CREATE YOTA OBJECT
myYota = yNo(youtube_url_from_arg)
# SEARCH SUBTITLES .SRT
srt_search_results = myYota.srt_search(search_term_from_arg, clip_length=clip_length_from_arg)

## OUTPUT ##
# PRINT RESULT
for item in srt_search_results:
    print(item.url)


#!/home/dd/anaconda3/bin/python
import re

def next_page(url):
    
    """Takes YouTube search page URL, returns next page URL."""
    
    search_terms = re.search('query=(.*?)(&|$)', url)
    first_url = 'https://www.youtube.com/results?search_query=grey+hats'
    second_url = 'https://www.youtube.com/results?search_query=grey+hats&page=2'
    
    check_for_page_no = re.search('&page=', url)
    if check_for_page_no:
        m = re.search('page=(\d{1,2})', url)
        page_no = m.group(1)
        new_page_no = str(int(page_no) + 1)
        new_url = url.replace('page=' + page_no, 'page=' + new_page_no)
    else:
        new_url = second_url.replace('grey+hats',search_terms.group(1))
        
    return(new_url)


#!/home/dd/anaconda3/bin/python
import re

def main(filename, yNo_function):

    def parse_yno_file_readlines(content):
        """parse .omm readlines list for yotas spanning 
        several lines, return list of yota strings."""

        parsed_lines = []
        for line in content:
                line = line.strip()
                if len(line) > 2:  # filter empty lines
                    m0 = re.search('^#', line)
                    if not m0: # filter comments
                        m = re.search('^\.', line)
                        if m: # concatinate multiple line yotas
                            parsed_lines[-1] += line
                        else:
                            parsed_lines.append(line)
        return(parsed_lines)

    with open(filename) as f:
        filelines = f.readlines()

    filelines2 = parse_yno_file_readlines(filelines)
    yNo = yNo_function

    if len(filelines2) > 1:
        mixtape = yNo(filelines2[0]) + yNo(filelines2[1])
    else:
        mixtape = yNo(filelines2[0])

    for line in filelines2[2:]:
        mixtape += yNo(line)

    return(mixtape.omm_oneline())




#!/home/dd/anaconda3/bin/python

# takes YouTube URL, matching .srt filename and search term(s),
# returns time code YouTube URLs of occurrences

import re, sys
from datetime import timedelta


def main(srt_file, search_term):
    """Takes .srt filename and search term, 
    returns YouTube time code URLs of occurrences."""

    f = open(srt_file,'r')
    content = f.readlines()
    f.close()
    
    new_content = []
    for line in content:
        newline = re.sub('<.+?>','', line)
        new_content.append(newline)

    def run_ad_time_parser():
        """Search a .srt file for time codes associated with search term."""
        result_list = []
        for i, line in enumerate(new_content):
            m = re.search(search_term, line)
            if m:
                result_list.append(new_content[i-1])

        return(result_list)

    result_list = run_ad_time_parser()

    def convert_time(time_str):
        m = re.search('^(\d\d):(\d\d):(\d\d)', time_str)
        try:
            hours = m.group(1)
        except:
            hours = 0
        try:
            minutes = m.group(2)
        except:
            minutes = 0
        try:
            seconds = m.group(3)
        except:
            seconds = 0
        
        
        # correct time code 2 seconds back
        current_time = timedelta(hours=int(hours), minutes=int(minutes), seconds=int(seconds))
        subtraction_time = timedelta(seconds=2)
        if subtraction_time < current_time:
            #print('current_time:', current_time)
            corrected_time = current_time - subtraction_time
        # in case of 0 or 1 seconds
        else:
            #print('correcting:', current_time)
            corrected_time = timedelta(seconds=1)  # NB: start from 1s minimum
        
        time_str = str(corrected_time)
        m = re.search('(\d{1,2}):(\d\d):(\d\d)', time_str)
        hours = m.group(1)
        minutes = m.group(2)
        seconds = m.group(3)

        return(hours, minutes, seconds)

    result_matrix = []
    for line in result_list:
        result_matrix.append(convert_time(line))

    try:
        m = re.search('\-([a-zA-Z0-9\-\_]{11})\.', srt_file)
        if m:
            base_url = 'https://www.youtube.com/watch?v=' + m.group(1)
    except ValueError:
            print('No YouTube hash matched in filename')

    urls_with_time_codes = []
    for tup in result_matrix:
        if int(tup[0]) > 0: # hours present
            new_url = base_url + '&t=' + str(tup[0]) + 'h'
        else:
            new_url = base_url + '&t='
        if int(tup[1]) > 0: # minutes present           
            new_url = new_url + str(tup[1]) + 'm'
        if int(tup[2]) > 0: # seconds present
            new_url = new_url + str(tup[2]) + 's'
            urls_with_time_codes.append(new_url)
    uniqued_results = []
    for item in urls_with_time_codes:
        if not item in uniqued_results:
            uniqued_results.append(item)
    return (uniqued_results[1:])  # NB: Very hacky, throwing away first faulty result

if __name__ == '__main__':

    result = main(sys.argv[1], sys.argv[2])
    for item in result:
        print(item)



import re
import subprocess
import requests
import json
import os
import uuid
import hashlib
import string
from . import cnf
from . import yota
from . import sh



# get version number from cnf.py
version_number = cnf.version_number


def check_cache(hash_to_check, version_number):
    """Use local cache information if present."""

    # check for existence of bitly hash cashe JSON
    if os.path.isfile('hashes_i_have_seen.json'):
        with open('hashes_i_have_seen.json', 'r') as f:
            hashesIHaveSeen = json.load(f)
    else:
        hashesIHaveSeen = {}
    if hash_to_check in hashesIHaveSeen:

        return(hashesIHaveSeen[hash_to_check])
    else:
        bitly_url = 'https://bit.ly/' + hash_to_check

        #header_str = 'MediaByte ' + version_number
        #headers = {'User-Agent': header_str}
        r = requests.get(bitly_url) #, headers=headers)

        # r = requests.get(bitly_url)
        parsed_url = r.url
        hashesIHaveSeen[hash_to_check] = parsed_url
        f = open("hashes_i_have_seen.json", "w")
        json.dump(hashesIHaveSeen, f)
        f.close()

        return(parsed_url)


def youtube_time_format(seconds):
    """Takes time in seconds, returns YouTube format time code."""

    hours = int(seconds / 3600)
    seconds = seconds - (hours * 3600)
    minutes = int(seconds / 60)
    rest = seconds - minutes * 60
    final_str = ''
    if hours:
        final_str += str(hours) + 'h'
    if minutes:
        final_str += str(minutes) + 'm'
    if seconds:
        final_str += str(rest) + 's'

    return(final_str)


class Link():
    """MediaByte Bit Link object."""

    def __init__(self, bitly_hash, tags=[], title=None):

        def parse_link():

            #headers = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 Safari/537.36'}

            new_url = check_cache(self.bitly_hash, version_number)

            return(new_url)

        self.bitly_hash = bitly_hash

        # NB: temporary solution, random first_name for Mixtape creation
        self.first_name = uuid.uuid1().hex[:3]

        if tags:
            self.tags = tags
        else:
            self.tags = None

        if title:
            title2 = title.replace("_", " ")
            self.title = title2
            if self.tags:
                if self.tags[0] == title:
                    self.tags.pop(0)

        else:
            self.title = None

        self.url = 'https://www.bitly.com/' + self.bitly_hash

        self.link = parse_link()

        if self.tags and self.title:
            self.html = '<a href="' + \
                str(self.link) + '" title="' + " ".join(tags) + \
                '">' + str(self.title) + '</a>'

        elif self.title:
            self.html = '<a href="' + \
                str(self.link) + '">' + str(self.title) + '</a>'

        else:
            self.html = '<a href="' + \
                str(self.link) + '" title="' + str(self.tags) + '"> MyLink</a>'


        # OMM formatting
        self.omm = 'b.' + self.bitly_hash

        if self.title:
            self.omm += '.' + self.title.replace(" ", "_")
        if self.tags and self.tags[0]:
            for tag in self.tags:
                self.omm += '.' + tag
        
        # if self.omm[-1] == '.':
        #     self.omm = self.omm[:-1]  # NB: Very hacky, removing trailing '.' dot (coming from where?)

        if self.title:
            self.html = '<a href="' + self.link + '">' + self.title + '</a>'
        else:
            self.html = '<a href="' + self.link + '">MyBit</a>'


    def play(self):
        """Open sample in tab in browser."""

        subprocess.Popen(['google-chrome', self.link])

    def vlc(self):
        """Play Link in VLC Player."""

        subprocess_str = 'vlc ' + self.link
        subprocess.Popen([subprocess_str], shell=True)

    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(Link):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)

    def __repr__(self):
        """Defines internal print() format (internal method)."""

        if self.title == None:
            self.title = 'MyBit'
            set_temp_title = True
        else: set_temp_title = False

        tag_string = "  ("                      # build tag string
        if self.tags:
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '

        tag_string += ')'

        result = ""
        if self.title:
            result = self.title + result

        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
            result += tag_string

        if set_temp_title:
            self.title == None

        return(result)

    def __str__(self):
        """Returns OMM format string (internal method)."""

        return(self.omm)

    def __add__(self, value):

        new_mixtape_obj = yota.Mixtape(self)
        new_mixtape_obj += value

        return(new_mixtape_obj)

    def iframe(self, width=360, pause=False, title=False, center=True):
        """Returns IFrame format."""

        url = '<center><iframe width="360" title="YYYYY" src="XXXXX" frameborder=0 allowfullscreen></iframe></center>'
        if width != 360:
            url = url.replace('360', str(width))

        newUrl = url.replace('XXXXX', self.link)
        if title:
            newUrl = newUrl.replace('YYYYY', self.title)
        else:
            newUrl = newUrl.replace('title="YYYYY"','')
        if pause:
            newUrl = newUrl.replace('autoplay=1', 'autoplay=0')
        if title:
            new_title = '<center><p>' + repr(self) + '<p>'
            newUrl = newUrl.replace('<center>', new_title)
        if not center:
            newUrl = newUrl[8:-9]

        return(newUrl)

    def update(self):
        """Updates sample 'url' and 'html' parameters."""

        self.url = 'https://www.youtube.com/embed/' + \
            self.youtube_hash + '?start=0' + '&rel=0' + '&autoplay=1'

        # OMM formatting
        self.omm = 'b.' + self.bitly_hash

        if self.title:
            self.omm += '.' + self.title.replace(" ", "_")
        if self.tags and self.tags[0]:
        #if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag

        self.html = '<a href="' + self.link + '">' + self.title + '</a>'


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)



class Mp3():
    """MediaByte Bit MP3 object."""

    def __init__(self, bitly_hash, tags=[], title=None, time_start=None, time_end=None):

        def parse_link():

            new_url = check_cache(self.bitly_hash, version_number)
            # r = requests.get(self.url, headers=headers)
            # new_url = r.url

            return(new_url)

        self.bitly_hash = bitly_hash

        self.first_name = uuid.uuid1().hex[:3]

        if tags:
            self.tags = tags
        else:
            self.tags = None

        if time_start:
            self.time_start = time_start
        else:
            self.time_start = None
        if time_end:
            self.time_end = time_end
        else:
            self.time_end = None

        if title:
            title2 = title.replace("_", " ")
            self.title = title2
            if self.tags:
                if self.tags[0] == title:
                    self.tags.pop(0)

        else:
            self.title = None

        self.url = 'https://www.bitly.com/' + self.bitly_hash

        self.link = parse_link()

        if self.tags and self.title:
            self.html = '<a href="' + \
                str(self.link) + '" title="' + " ".join(tags) + \
                '">' + str(self.title) + '</a>'

        elif self.title:
            self.html = '<a href="' + \
                str(self.link) + '">' + str(self.title) + '</a>'

        else:
            self.html = '<a href="' + \
                str(self.link) + '" title="' + str(self.tags) + '"> MyMp3</a>'


        # OMM formatting
        self.omm = 'b.' + self.bitly_hash
        if self.time_start:
            self.omm += '.' + str(youtube_time_format(self.time_start))

        if self.title:
            self.omm += '.' + self.title.replace(" ", "_")
        if self.tags and self.tags[0]:
        #if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag
        if self.time_end:
            self.omm += '.' + str(youtube_time_format(self.time_end))

        if self.title:
            self.html = '<a href="' + self.link + '">' + self.title + '</a>'
        else:
            self.html = '<a href="' + self.link + '">MyBit</a>'

    def play(self):
        """Open sample in tab in browser."""

        subprocess.Popen(['google-chrome', self.link])

    def vlc(self):
        """Play Mp3 in VLC Player."""

        if self.time_start and self.time_end:
            parsed_link = self.link
            start_time_str = '--start-time=' + \
                str(self.time_start) + ' ' + \
                '--stop-time=' + str(self.time_end)
            subprocess_str = 'vlc ' + parsed_link + ' ' + start_time_str

            p = subprocess.Popen([subprocess_str], shell=True)

        elif self.time_start:
            parsed_link = self.link
            start_time_str = '--start-time=' + str(self.time_start)
            subprocess_str = 'vlc ' + parsed_link + ' ' + start_time_str

            p = subprocess.Popen([subprocess_str], shell=True)

        else:
            subprocess_str = 'vlc ' + parsed_link

            p = subprocess.Popen([subprocess_str], shell=True)

    def methods(self):
        """Show methods and parameters."""

        result = []
        for item in dir(Mp3):
            if item.startswith('__'):
                pass
            else:
                result.append(item)

        return(result)

    def __repr__(self):
        """Defines internal print() format (internal method)."""

        if self.title == None:
            self.title = 'MyBit'
            set_temp_title = True
        else: set_temp_title = False

        tag_string = "  ("                      # build tag string
        if self.tags:
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '

        tag_string += ')'

        result = ""
        if self.title:
            result = self.title + result

        if self.tags:
            tag_string = "  ("                      # build tag string
            for i, tag in enumerate(self.tags):
                tag_string += str(tag)
                if (i + 1) < len(self.tags):
                    tag_string += ', '
            tag_string += ')'
            result += tag_string


        if set_temp_title:
            self.title == None

        return(result)

    def __str__(self):
        """Returns OMM format string (internal method)."""

        return(self.omm)

    def __add__(self, value):

        new_mixtape_obj = yota.Mixtape(self)
        new_mixtape_obj += value

        return(new_mixtape_obj)

    def __len__(self):
        """Returns clip length in seconds."""

        own_length_in_seconds = self.time_end - self.time_start

        return(own_length_in_seconds)

    def iframe(self, width=360, pause=False, title=False, center=True):
        """Returns IFrame format."""

        url = '<center><iframe width="360" title="YYYYY" src="XXXXX" frameborder=0 allowfullscreen></iframe></center>'
        if width != 360:
            url = url.replace('360', str(width))

        newUrl = url.replace('XXXXX', self.link)
        newUrl = newUrl.replace('YYYYY', self.title)
        if pause:
            newUrl = newUrl.replace('autoplay=1', 'autoplay=0')
        if title:
            new_title = '<center><p>' + repr(self) + '<p>'
            newUrl = newUrl.replace('<center>', new_title)
        if not center:
            newUrl = newUrl[8:-9]

        return(newUrl)

    def update(self):
        """Updates sample 'url' and 'html' parameters."""

        self.omm = 'b.' + self.bitly_hash

        if self.time_start:
            self.omm += '.' + youtube_time_format(self.time_start)

        if self.title:
            self.omm += '.' + self.title.replace(" ", "_")
        if self.tags and self.tags[0]:
        #if self.tags:
            for tag in self.tags:
                self.omm += '.' + tag

        if self.time_start and self.time_end:
            self.omm += '.' + youtube_time_format(self.time_end)

        self.html = '<a href="' + self.link + '">' + self.title + '</a>'

    def time(self):
        """Show sample length in 1h2m3s format."""

        own_length_in_seconds = self.time_end - self.time_start

        youtube_format_str = youtube_time_format(own_length_in_seconds)

        return youtube_format_str


    def hash(self):
        m = hashlib.sha256()
        m.update(self.omm.encode())
        temp_hash = m.hexdigest()
        for i, char in enumerate(temp_hash):
            if char in string.ascii_letters:
                calculated_hash = temp_hash[i:i+11]
                return(calculated_hash)




class Stream:
    """MediaByte Bit Stream object."""

    def __init__(self, bitly_hash, tags=[], title=None, time_start=None, time_end=None):

        self.bitly_hash = bitly_hash

        if time_start:
            self.time_start = time_start
            if time_end:
                self.time_end = time_end
            else:
                self.time_end = ""
        else:
            self.time_start = ""

        if title:
            self.title = title
        else:
            self.title = ""        

        if tags:
            self.tags = tags
        else:
            self.tags = []

        
        self.omm = "b." + self.bitly_hash
        if self.title:
            self.omm += "." + self.title
        self.omm += ".m3u8"
        if self.time_start:
            self.omm += "." + str(self.time_start)
        

        if not 'm3u8' in tags:
            raise ValueError ('Bit.Stream object needs to have the tag \'m3u8\'')

        def parse_link():
            url = "https://bitly.com/" + bitly_hash
            r = requests.get(url)
            new_url = r.url
        
            return(new_url)

        self.link = parse_link()
        
        self.url = "https://bit.ly/" + self.bitly_hash

        def hash():
            m = hashlib.sha256()
            m.update(self.omm.encode())
            temp_hash = m.hexdigest()
            for i, char in enumerate(temp_hash):
                if char in string.ascii_letters:
                    calculated_hash = temp_hash[i:i+11]
                    return(calculated_hash)
            
        self.hash = hash()

    def __repr__(self):
        """Defines internal print() format (internal method)."""

        if self.title == "":
            title = 'MyStream'
        else:
            title = self.title

        result = title


        return result        


    def __str__(self):
        
        return self.omm

    def vlc(self):
        if self.time_start:
            time_str = '--start-time=' + str(self.time_start)
            sh.ell('vlc', self.url, time_str)
        else:
            sh.ell('vlc', self.url)

    def download_m3u8(self):
        """Download .m3u8 in the browser"""
        sh.ell('xdg-open',self.url)

    
    
    # def hash(self):
    #     m = hashlib.sha256()
    #     m.update(self.omm.encode())
    #     temp_hash = m.hexdigest()
    #     for i, char in enumerate(temp_hash):
    #         if char in string.ascii_letters:
    #             calculated_hash = temp_hash[i:i+11]
    #             return(calculated_hash)


from . import bit
from . import yota
from . import cnf

from pydub import AudioSegment

import urllib.request
import os
import wget
import glob
import re


def cache_or_download(sampleObj):
    
    """Takes Sample, checks file cache, 
    downloads if needed, returns filename."""
    
    cache_result_found, filename = check_cache(sampleObj)
    if cache_result_found:
        print(f'using file from cache {filename}')
        return(filename)

    else:
        download_mp3(sampleObj)
        m = re.search('[^\/]+?mp3$', sampleObj.link)
        filename = m.group(0)
        return filename

def check_cache(sampleObj):
    filenames = get_sorted_mp3_filenames()
    m = re.search('[^\/]+?mp3$', sampleObj.link)
    sample_filename = m.group(0)
    for item in filenames:
        m = re.search(sample_filename, item)
        if m:
            return True, sample_filename
    return False, ""

def download_mp3(sampleObj):
    # download MP3 file
    print(f'downloading {sampleObj.link}')
    wget.download(sampleObj.link, cnf.package_path)

def cut_sample(sampleObj, filename, keep_original=False):
    """Cut Sample, save Sample MP3."""
    
    # load MP3 into memory (not fast, seconds)
    song = AudioSegment.from_mp3(filename)
    # cut sample from MP3
    cut_sample = song[sampleObj.time_start*1000:sampleObj.time_end*1000]
    try:
        len(sampleObj.title) > 0
    # set default bit title
    except:
        sampleObj.title = 'MyBit'
    clip_filename = sampleObj.title + '_' + str(sampleObj.time_start) + '_' + str(sampleObj.time_end) + '.mp3'
    # write sample to file
    cut_sample.export(clip_filename, format="mp3")
    print(f'sample {clip_filename} successfully cut...')
    
    if keep_original == False:
        os.remove(filename)
        print(f'original file {filename} deleted')

def get_sorted_mp3_filenames():
    """Returns MP3 filename list in order of creation."""
    files_path = cnf.package_path + cnf.os_sep + '*'
    files = sorted(
        glob.iglob(files_path), key=os.path.getctime) 
    mp3_filenames = []
    for filename in files:
        if str(filename).endswith('.mp3'):
            mp3_filenames.append(filename)
    return(mp3_filenames)

def splice_samples(mix_filename, transition_sound=False):
    filenames = get_sorted_mp3_chapters()
    first_clip = AudioSegment.from_mp3(filenames[0])
    if transition_sound:
        transition_clip = AudioSegment.from_mp3(transition_sound)
    for i, item in enumerate(filenames[1:]):
        if transition_sound:
            first_clip += transition_clip
        next_clip = AudioSegment.from_mp3(item)
        first_clip += next_clip
    first_clip.export(mix_filename, format="mp3")
    print(f'successfully wrote file {mix_filename}')

def get_sorted_mp3_chapters():
    """Returns MP3 filename list in order of creation."""
    files_path = os.path.join('.', '*')
    files = sorted(
        glob.iglob(files_path), key=os.path.getctime) 
    mp3_filenames = []
    for filename in files:
        if str(filename).endswith('.mp3'):
            mp3_filenames.append(filename)
    return(mp3_filenames)

def main(sampleObj, keep_original=False, splice=False):

    if isinstance(sampleObj, bit.Mp3):
        filename = cache_or_download(sampleObj)
        if filename and len(filename) > 0:
            filenames = get_sorted_mp3_filenames()
            filename = filenames[0]
            cut_sample(item, filename, keep_original=keep_original)

    elif isinstance(sampleObj, yota.Mixtape):
        for item in sampleObj:
            filename = cnf.package_path + cnf.os_sep + cache_or_download(item)
            cut_sample(item, filename, keep_original=keep_original)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ ]

setup_requirements = [ 'youtube-dl','requests','pydub']

test_requirements = [ ]

setup(
    description="temp",
    install_requires=requirements,
    include_package_data=True,
    name='mb',
    packages=['mb'],
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    version='0.8.9.6',
    zip_safe=False,
)



import sys

def get_site_packages_path():
    """"Returns the first sys.path 'site-packages' or 'dist-packages' match."""
    #print(f'sys.path: {sys.path}')
    for path in sys.path: 
        if 'site-packages' in path or 'dist-packages' in path:
            if flask_server == False:  # client execution environment
                if '.local' not in path: # so exclude .local result from sys.path
                    new_path = path + os_sep + 'mediabyte'
                    return new_path
            else: # server execution, so don't exclude .local result from sys.path
                new_path = path + os_sep + 'mediabyte'
                return(new_path) 

def get_os_file_separator():
    """Returns OS-dependent file separator character (Windows or other)."""
    if platform == 'win32':
        os_sep = '\\'
    else:
        os_sep = '/'
    return(os_sep)


# hard-coded canonical package version number

version_number = 'v0.8.9.6-pro'

summary = """Online Media Metadata (OMM) is an original concept to succinctly reference online media ressources in mediabytes - that is, by platform-specific hash or bitly link hash, with optional explicit tags and title (a title is a title-cased tag) and YouTube format time code(s) for YouTube videos and MP3s."""


# constants for use in modules

flask_server = False
platform = sys.platform
os_sep = get_os_file_separator()
package_path = get_site_packages_path()
srt_folder_path = package_path + os_sep + 'srt'
hash_dict_path = package_path + os_sep + 'files' + os_sep + 'hash_dict.json'
filters_path = package_path + os_sep + 'files' + os_sep + 'filters.json'



